/**
 ** main.c
 **
 ** Main program file for dsPIC Bootloader
 **
 ** (c) 2008 Cambridge Micro Engineering Ltd.
 **
 ** Software License Agreement
 **
 ** The software supplied herewith is (c) Cambridge Micro Engineering
 ** (the "Company") and is intended and supplied to you, the Company's
 ** customer, for use solely and exclusively under the terms of the
 ** software licence.
 **
 ** The software is owned by the Company and/or its supplier, and is
 ** protected under applicable copyright laws. All rights are reserved.
 ** Any use in violation of the foregoing restrictions may subject the
 ** user to criminal sanctions under applicable laws, as well as to
 ** civil liability for the breach of the terms and conditions of this
 ** license.
 **
 ** Derived from example code (c) Microchip where applicable
 **/

#include <p33fxxxx.h>
#include <stdio.h>
#include <ctype.h>
#include <uart.h>

#include "serial.h"
#include "version.h"

/*****************************************************************************
*   Configuration Variables
******************************************************************************/
_FBS(RBS_NO_RAM & BSS_NO_FLASH & BWRP_WRPROTECT_OFF)
_FSS(RSS_NO_RAM & SSS_NO_FLASH & SWRP_WRPROTECT_OFF)
_FGS(GSS_OFF    & GCP_OFF      & GWRP_OFF)
_FOSCSEL(FNOSC_FRCPLL & IESO_ON)
_FOSC(FCKSM_CSDCMD & OSCIOFNC_ON & POSCMD_NONE)
_FWDT(FWDTEN_OFF & WINDIS_OFF & WDTPRE_PR128 & WDTPOST_PS32768)
_FPOR(FPWRT_PWR128)

#define COMMAND_NACK     0x00
#define COMMAND_ACK      0x01
#define COMMAND_READ_PM  0x02
#define COMMAND_WRITE_PM 0x03
#define COMMAND_WRITE_CM 0x07
#define COMMAND_RESET    0x08
#define COMMAND_READ_ID  0x09

#define RESET_VECTOR_ADDR   0x0000
#define BOOTLOAD_START_ADDR 0x0400      /* Start of bootload code */
#define BOOTLOAD_WORD_ADDR  0x1000      /* address of byte to be checked by bootload software  */
#define APP_START_ADDR      0x2000      /* Keyboard application start address  */
#define EEPROM_START_ADDR   0x15000     /* EEPROM start address  */

#define HEX_ADD_DVDR        2

#define PM_ROW_SIZE 64 * 8
#define PM_PAGE_SIZE ( PM_ROW_SIZE * 3 )
#define CM_ROW_SIZE 8
#define CONFIG_WORD_SIZE 1

#define PM_ROW_ERASE        0x4042
#define PM_ROW_WRITE        0x4001
#define CONFIG_WORD_WRITE   0X4000

#define FCY             79227500/2    //40000000
#define BAUDRATE        57600
#define BRGVAL          ((FCY/BAUDRATE)/16)-1

#define INPUT   1
#define OUTPUT  0

#define BSTRAP_A_IO   PORTGbits.RG8     /* Use transpose keys to force bootstrap */
#define BSTRAP_A_DIR  TRISGbits.TRISG8

#define BSTRAP_B_IO   PORTGbits.RG9     /* Use transpose keys to force bootstrap */
#define BSTRAP_B_DIR  TRISGbits.TRISG9

#define BSTRAP_POLARITY  0              /* A zero on this pin indicates Bootstrap mode */

#define STATUS_BYTE         0xF0        /* from B0  */
#define EOX_BYTE            0xF7        /* from 0x00 */

#define SPACE               0x20
#define DASH                0x2D
#define COLON               0x3A

/* MAnufacturers MIDI id bytes */
#define ID1                 0x00
#define ID2                 0x20
#define ID3                 0x6D

/* response codes  */
#define CODE_ACK            0x7F
#define CODE_NACK           0x7E
#define CODE_VER_NO         0x7D
#define CODE_SER_NO         0x7C
#define CODE_MODE           0x7B

/* System Message  */
#define USB_BOOT            0xF9

#define NUM_SERIAL_BYTES    0x20        /* 32 */

#define EE_STATUS_ADDRESS   0x0000
#define EE_SERIAL_ADDRESS   0x0002

#define PM_WORD_WRITE 0x4003

#define TX_DELAY_COUNT  0xFFFF
#define POR_DELAY_COUNT 0x005FFFFF

typedef enum
{
    WAIT_COMMAND,
    WAIT_ID1,
    WAIT_ID2,
    WAIT_ID3,
    WAIT_SYSCOM,
    WAIT_SER_NO_DATA,
    WAIT_EOX,
    WAIT_DNLD,

}tCommandStatus;

typedef enum
{
    WAIT_PR_RECMARK,    /* the following cases deal with data in from intel hex file  */
    WAIT_PR_RECLEN,
    WAIT_PR_OFFSET_LOW,
    WAIT_PR_OFFSET_HI,
    WAIT_PR_RECTYP,
    WAIT_PR_DATA,
    WAIT_PR_CHKSUM
}tDnldStatus;

typedef enum
{
    RT_DATA_REC,
    RT_EOF_REC,
    RT_ESA_REC,
    RT_SSA_REC,
    RT_ELA_REC,
    RT_SLA_REC
} tRecType;

typedef enum
{
    SYSCOM_NONE,
    SYSCOM_TESTMODE,
    SYSCOM_APP_ENABLE,
    SYSCOM_APP_DISABLE,
    SYSCOM_BOOTLOAD,          // Changed from 0x78,
    SYSCOM_SERIAL_WRITE,
    SYSCOM_VERSION_READ,
    SYSCOM_SERIAL_READ,
    SYSCOM_MODE_READ,
    SYSCOM_ERASE,
    SYSCOM_PROG
}   tCommandPending;

typedef enum
{
    SYSSTATUS_BOOTMODE,
    SYSSTATUS_TESTMODE,
    SYSSTATUS_APP_ENABLED,
    SYSSTATUS_APP_DISABLED,
    SYSSTATUS_ILLEGAL = 0x00FF

}   tSystemStatus;

typedef enum
{
    RESPONSE_ACK,
    RESPONSE_NACK,
    RESPONSE_VER_NO,
    RESPONSE_SER_NO,
    RESPONSE_MODE
}   tResponseType;


byte iSerialNumber;
char SerialNumber[NUM_SERIAL_BYTES];

typedef unsigned char  Byte;
typedef short          Word16;
typedef unsigned short UWord16;
typedef long           Word32;
typedef unsigned long  UWord32;

typedef union tuReg32
{
    UWord32 Val32;

    struct
    {
        UWord16 LW;
        UWord16 HW;
    } Word;

    char Val[4];
} uReg32;

typedef struct {
    Byte        RecLength;
    UWord16     Offset;
    UWord16     ULBA;
    UWord16     LLBA;
    tRecType    Type;
    Byte        Data [256];
    Byte        ChkSum;
} tHexRecord;

void PutChar(char);
void GetChar(char *);
void WriteBuffer(char *, int);
void ReadPM(char *, uReg32);
void WritePM(char *, uReg32);
void GoResetDevice(void);

typedef struct
{
    UWord16     UAddr;
    UWord16     LAddr;
    word iBuffer;
    char Buffer[PM_PAGE_SIZE];
} tProgPage;

/* Prototypes */
void SendResponse(tResponseType ResponseType );
void InterruptEnable (void);
void InterruptDisable (void);
void ClearBootloadVector(void);
void EraseProgramMemory(void);
byte ConvertASCII ( byte ch);

/* Externals */

extern dword ReadLatch(word, word);
extern dword WriteLatch(word, word, word, word);
extern void ResetDevice(void);
extern void ResetBootstrap(void);
extern void WriteMem(word);
extern void Erase(word, word ,word);

tHexRecord  HexRecord;
tProgPage   ProgPage;

int main(void)

{
    byte InData, TempData;
    tCommandStatus CommandStatus = WAIT_COMMAND;
    tCommandPending CommandPending = SYSCOM_NONE;
    tDnldStatus DnldStatus = WAIT_PR_RECMARK;

    uReg32 SourceAddr, TempData1, TempData2;
    uReg32 Delay;

    /* Configure CLKDIV for Maximum Fcy */

    /*
    OSCILLATOR RULES:
    A) Output from the PLLPRE "MUST" be in the range of (0.8-8.0 MHz) at all times
    B) Output from the VCO, (PLLDIV), "MUST" be in the range of (100-200 MHz) at all times
    C) Output from the PLLPOST "MUST" be in the range of (12.5-80 MHz) at all times
    D) Violating any rule while the PLL is active and the oscillator will become unstable
       and is not guaranteed to return to normal even if values are later corrected in
       CLKDIV & PLLFBD to be within range.

    Using internal fast RC @ 7.37MHz
    At power on:
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 184.25Mhz = OK
      //NOTE:Output from the PLLPOST = 46.0625Mhz = OK


    Default POR values:
    PLLPRE = 0; ...........(0+2) = 2
    PLLDIV = 0x30 = 48;....(48+2)= 50
    PLLPOST = 1; ..........(4)

    Final configuration
    PLLPRE = 0; ...........(0+2) = 2
    PLLDIV = 0x26 = 38;....(38+2)= 40
    PLLPOST = 0; ..........(2)
    */

    CLKDIVbits.PLLPRE  = 0; /* Pre Divide by 2 */
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 184.25Mhz = OK
      //NOTE:Output from the PLLPOST = 46.0625Mhz = OK

    PLLFBDbits.PLLDIV  = 41;    /* PLL Multiply by 43  */
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 147.4Mhz = OK
      //NOTE:Output from the PLLPOST = 46.0625Mhz = OK

    CLKDIVbits.PLLPOST = 0; /* Post Divide by 2     */
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 147.4Mhz = OK
      //NOTE:Output from the PLLPOST = 79.2275Mhz = OK

    CLKDIVbits.FRCDIV  = 0; /* FRC divide by 1 */
      //FRCDIV bits are don't care if PLL mode enabled.
      //FRCDIV bits are only important if your configuration words are something like
      //  _FOSC( FCKSM_CSDCMD & OSCIOFNC_ON & POSCMD_NONE);
      //  _FOSCSEL( FNOSC_FRC & IESO_OFF & TEMP_OFF);

    /* need to tune internal oscillator up by 1.125% */
    OSCTUNbits.TUN = 3;   /*  !!!!  */

    //Wait until PLL has re-locked
    while(OSCCONbits.LOCK != 1);

    /* Check for  transposition keys pressed */
    BSTRAP_A_DIR = INPUT;
    BSTRAP_B_DIR = INPUT;

    /* if both keys are pressed */
    if ( (BSTRAP_A_IO == 0) && (BSTRAP_B_IO == 0) )
    {
        SERIALInit();

        /* Assert CTS   */
        LATFbits.LATF13 = 1;
        TRISFbits.TRISF13 = 0;     /* Output */

        TRISFbits.TRISF12 = 1;         /* Input */
        while ( PORTFbits.RF12 == 0); /* Wait for RTS  */

        /* delay  */
        for (Delay.Val32 = 0; Delay.Val32 < POR_DELAY_COUNT; Delay.Val32++)
        {
            Nop();
        }

        /* send system message */
        WriteUART2((unsigned int) USB_BOOT  );

        /* wait for transmit to complete  */
        while (!U2STAbits.TRMT);

        /* delay  */
        for (Delay.Val32 = 0; Delay.Val32 < TX_DELAY_COUNT; Delay.Val32++)
        {
            Nop();
        }

        /* wait for reset from USB chip*/
        while (1)
            Sleep();

    }


    /* Check if program memory valid (i.e. reset vector has been erased) */
    SourceAddr.Val32 = BOOTLOAD_WORD_ADDR;

    Delay.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

    /* The program memory at BOOTLOAD_WORD_ADDR holds the "key" to program memory status.
       If the contents of BOOTLOAD_WORD_ADDR are blank (0xff) then the memory is either unprogrammed
       or has been erased to force a boot strap. */

    if( (Delay.Val32 & 0x000000FF) != 0xFF) // is unprogrammed or erased
    {
        ResetDevice();
    }

    /* Now the only escape is to reprogram the program memory and clear BOOTLOAD_WORD_ADDR to non-zero */
    /* If the keyboard is disconnected at this point it will powerup in bootstrap mode. Thus we must support */
    /* all SYSCOM commands to allow the test app to recognise the mode. */

    InterruptDisable ();

    /* Initialise Serial Port */
    SERIALInit();

    /* Force CTS low - don't bother with flow control here */
    LATFbits.LATF13 = 0;
    TRISFbits.TRISF13 = 0;

    /* Wait for USB's TX shift register to empty */
    for (Delay.Val32 = 0; Delay.Val32 < TX_DELAY_COUNT; Delay.Val32++)
    {
        Nop();
    }

    /* Dummy read to clear RX reg */
    InData = U2RXREG;

    /* Force CTS high */
    LATFbits.LATF13 = 1;

    /* Send ACK to signify we're alive */
    SendResponse( RESPONSE_ACK );

    TempData = 0;

    /* Start main loop (no escape) */
    while(1)
    {
        /* Test for serial input */

        if (!SERIALInBufferIsEmpty())
        {
            InData = SERIALReadByte();

            switch(CommandStatus)
            {
                case WAIT_COMMAND:   /* waiting for a command string to start i.e status byte  */
                    if (InData == STATUS_BYTE)  /* valid command start byte */
                        CommandStatus = WAIT_ID1;    /* wait for valid id */
                break;

                case WAIT_ID1:   /* waiting for first Id byte  */
                    if (InData == ID1)  /* valid ID1 byte */
                        CommandStatus = WAIT_ID2;    /* wait for valid id */
                    else
                        CommandStatus = WAIT_COMMAND;    /* back to beginning */
                break;

                case WAIT_ID2:   /* waiting for second Id  byte  */
                    if (InData == ID2)  /* valid second Id  byte */
                        CommandStatus = WAIT_ID3;    /* wait for valid id */
                    else
                        CommandStatus = WAIT_COMMAND;    /* back to beginning */
                break;

                case WAIT_ID3:   /* waiting for Id string to start i.e status byte  */
                    if (InData == ID3)  /* valid third Id byte */
                        CommandStatus = WAIT_SYSCOM;    /* wait for valid command */
                    else
                        CommandStatus = WAIT_COMMAND;    /* back to beginning */
                break;

                case WAIT_SYSCOM:   /* Waiting for a valid command byte */
                {
                    switch(InData)
                    {
                        case SYSCOM_ERASE:   /*  Erase command receieved  */
                            CommandPending = SYSCOM_ERASE;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_PROG:   /*  Program command receieved  */
                            CommandPending = SYSCOM_PROG;
                            CommandStatus = WAIT_DNLD;
                            DnldStatus = WAIT_PR_RECMARK;

                            /* Initialise HexRecord */
                            HexRecord.Offset = 0;
                            HexRecord.RecLength = 0;
                            HexRecord.ULBA   = 0;       /* Zero extended address (LBA) */
                            HexRecord.LLBA   = 0;
                            HexRecord.ChkSum = 0;

                            /* Initialise Programming buffer  */
                            for ( ProgPage.iBuffer = 0; ProgPage.iBuffer < PM_PAGE_SIZE; ProgPage.iBuffer++)
                                ProgPage.Buffer[ProgPage.iBuffer] = 0xFF;

                            ProgPage.LAddr = 0;
                            ProgPage.UAddr = 0;
                            ProgPage.iBuffer = 0;
                        break;

                        case SYSCOM_VERSION_READ:   /* Read Version command receicved */
                            CommandPending = SYSCOM_VERSION_READ;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_MODE_READ:      /* Read Mode command received */
                            CommandPending = SYSCOM_MODE_READ;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_BOOTLOAD:       /* Bootload command receieved */
                            CommandPending = SYSCOM_BOOTLOAD;
                            CommandStatus = WAIT_EOX;
                        break;

                        default:    /* Valid command not received, start wait for command again */
                            CommandPending = SYSCOM_NONE;
                            CommandStatus = WAIT_EOX;
                        break;

                    } /*    switch(InData)  */
                }

                break;  /* case WAIT_SYSCOM */

                case    WAIT_DNLD:  /* Wait for download file */

                    if ((InData == COLON)||(isxdigit(InData)))
                    {
                        switch(DnldStatus)
                        {
                            UWord16 iData = 0;
                            UWord16 TempData;
                            byte    ByteCnt;
                            /* Wait PR cases are to read in and interpret Intel Hex files */

                            /*  :        AA       BBBB    CC        nn nn nn nn nn nn  KK
                                RECMARK  RECLEN   OFFSET  RECTYP    DATA               CHKSUM
                            */

                            /*
                                RECTYP is encoded as follows:

                                00 Data Record
                                0l End of File Record
                                02 Extended Segment Address Record
                                03 Start Segment Address Record
                                04 Extended Linear Address Record
                                05 Start Linear Address Record
                            */

                            case     WAIT_PR_RECMARK:   /* colon or Ascii 0x3A  */
                                if (InData == COLON)
                                {
                                    DnldStatus = WAIT_PR_RECLEN;
                                    ByteCnt = 0;
                                }
                            break;  /* case WAIT_PR_RECMARK */

                            case     WAIT_PR_RECLEN: /* read and convert two bytes of data */
                                if (ByteCnt == 0)
                                {
                                    HexRecord.RecLength = (ConvertASCII(InData)<<4);
                                    ByteCnt++;
                                }
                                else
                                {
                                    HexRecord.RecLength += ConvertASCII(InData);
                                    HexRecord.ChkSum = HexRecord.RecLength;
                                    DnldStatus = WAIT_PR_OFFSET_HI;
                                    ByteCnt = 0;
                                }
                            break;  /* case WAIT_PR_RECLEN */

                            case    WAIT_PR_OFFSET_HI:  /* read and convert first 2 bytes of offset */
                                if (ByteCnt == 0)
                                {
                                    HexRecord.Offset = (ConvertASCII(InData)<<4);
                                    ByteCnt++;
                                }
                                else
                                {
                                    HexRecord.Offset += ConvertASCII(InData);
                                    HexRecord.ChkSum += HexRecord.Offset;
                                    DnldStatus = WAIT_PR_OFFSET_LOW;
                                    ByteCnt = 0;
                                }
                            break;  /* case WAIT_PR_OFFSET_HI */

                            case    WAIT_PR_OFFSET_LOW: //* read and convert second 2 bytes of offset */
                                if (ByteCnt == 0)
                                {
                                    TempData = (ConvertASCII(InData)<<4);
                                    ByteCnt++;
                                }
                                else
                                {
                                    TempData += ConvertASCII(InData);
                                    HexRecord.ChkSum += TempData;
                                    HexRecord.Offset = (HexRecord.Offset << 8) + TempData;
                                    DnldStatus = WAIT_PR_RECTYP;
                                    ByteCnt = 0;
                                }
                            break;  /* case WAIT_PR_OFFSET_LOW */

                            case    WAIT_PR_RECTYP: /* read and convert two bytes of data */
                                if (ByteCnt == 0)
                                {
                                    TempData = (ConvertASCII(InData)<<4);
                                    ByteCnt++;
                                }
                                else
                                {
                                    TempData += ConvertASCII(InData);
                                    HexRecord.ChkSum += TempData;
                                    HexRecord.Type = (tRecType)TempData;

                                    if (HexRecord.Type == RT_EOF_REC)
                                        DnldStatus = WAIT_PR_CHKSUM;
                                    else
                                        DnldStatus = WAIT_PR_DATA;

                                    ByteCnt = 0;
                                    iData = 0;
                                }
                            break;  /* case WAIT_PR_RECTYP*/

                            case    WAIT_PR_DATA:
                                /* read in and convert two bytes of data for record length number of data bytes  */
                                if (iData < HexRecord.RecLength)
                                {
                                    if (ByteCnt == 0)
                                    {
                                        TempData = (ConvertASCII(InData)<<4);
                                        ByteCnt++;
                                    }
                                    else
                                    {
                                        TempData += ConvertASCII(InData);
                                        HexRecord.ChkSum += TempData;
                                        ByteCnt = 0;

                                        HexRecord.Data[iData++] = TempData;

                                        /* check for end of data */
                                        if (iData == HexRecord.RecLength)
                                        {
                                            DnldStatus = WAIT_PR_CHKSUM;
                                            ByteCnt = 0;
                                        }

                                    }
                                }
                            break;  /* case WAIT_PR_DATA */

                            case    WAIT_PR_CHKSUM:

                                if (ByteCnt == 0)
                                {
                                    TempData = (ConvertASCII(InData)<<4);
                                    ByteCnt++;
                                }
                                else
                                {
                                    TempData += ConvertASCII(InData);
                                    ByteCnt = 0;

                                    TempData = -TempData;

                                    if (HexRecord.ChkSum == (byte)TempData)
                                    {
                                        switch (HexRecord.Type)
                                        {
                                                case    RT_DATA_REC:

                                                /*  dissassert CTS  to allow time to manipulate Hexrecord Data into buffer  */
                                                LATFbits.LATF13 = 0;

                                                /* Calculate full address for hex record */
                                                SourceAddr.Val32 = ((UWord32)ProgPage.UAddr << 16) + (HexRecord.Offset/HEX_ADD_DVDR);

                                                /* if the  full address is before boot load memory or in Application memory  */
                                                if  (( SourceAddr.Val32 <  BOOTLOAD_START_ADDR )
                                                     ||
                                                     ((SourceAddr.Val32 >=  APP_START_ADDR )
                                                     &&
                                                     (SourceAddr.Val32 <  EEPROM_START_ADDR ))
                                                    )
                                                {
                                                    /* if ProgPage.LAddr is zero and the buffer index is zero check against the data record address  */
                                                    if (( ProgPage.LAddr == 0)&& (ProgPage.iBuffer == 0))
                                                    {
                                                        ProgPage.LAddr = HexRecord.Offset/HEX_ADD_DVDR;
                                                    }

                                                     /*  check HexRecord (byte) address is the same as the (byte) address that buffer will be sent to plus buffer offset
                                                        adjusted to allow for loss of discarded bytes */
                                                    if ( HexRecord.Offset == ((ProgPage.LAddr * HEX_ADD_DVDR) + ((ProgPage.iBuffer*4)/3)))
                                                    {

                                                        /* place data from HexRecord in next available position in Program Page Buffer discarding every fourth byte (zero padding)   */
                                                        for (   iData = 0; iData < HexRecord.RecLength; iData++)
                                                        {
                                                            if ( (iData + 1) % 4)
                                                            {
                                                                ProgPage.Buffer[ProgPage.iBuffer++] = HexRecord.Data[iData];
                                                            }
                                                        }

                                                        /*  if the Program Page buffer is full  */
                                                        if ( ProgPage.iBuffer >= PM_PAGE_SIZE )
                                                        {

                                                            /* if lower address is still zero the data coming in must be the vector tables  */
                                                            if ( ProgPage.LAddr == 0)
                                                            {
                                                                /* read the reset vector from the flash  memory */
                                                                SourceAddr.Val32 = RESET_VECTOR_ADDR;
                                                                TempData1.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);
                                                                SourceAddr.Val32 = RESET_VECTOR_ADDR + 2;
                                                                TempData2.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

                                                                /*  Place the reset vector data (3 bytes) and the following 3 bytes into the buffer at the first 6 locations */
                                                                ProgPage.Buffer[0] =  TempData1.Val[0];
                                                                ProgPage.Buffer[1] =  TempData1.Val[1];
                                                                ProgPage.Buffer[2] =  TempData1.Val[2];
                                                                ProgPage.Buffer[3] =  TempData2.Val[0];
                                                                ProgPage.Buffer[4] =  TempData2.Val[1];
                                                                ProgPage.Buffer[5] =  TempData2.Val[2];
                                                            } /* if lower address is still zero the data coming in must be the vector tables  */

                                                          /*  program the memory  */
                                                            SourceAddr.Word.HW = ProgPage.UAddr;
                                                            SourceAddr.Word.LW = ProgPage.LAddr;
                                                            WritePM( ProgPage.Buffer, SourceAddr);

                                                            /*  reset buffer address and pointer */
                                                            for ( ProgPage.iBuffer = 0; ProgPage.iBuffer < PM_PAGE_SIZE; ProgPage.iBuffer++)
                                                                ProgPage.Buffer[ProgPage.iBuffer] = 0xFF;

                                                            ProgPage.LAddr = 0; /* do not reset UAddr - this is dealt with a RT_ELA_REC */
                                                            ProgPage.iBuffer = 0;
                                                        }   /*  if the Program Page buffer is full  */

                                                    }     /*  check HexRecord (byte) address is the same as the (byte) address that buffer will be sent to plus buffer offset */
                                                      /* else if HexRecord (byte) address is greater than the (byte) address buffer will be sent to plus buffer offset adjusted to
                                                        allow for loss of discarded bytes (the assumption is that all data will be contiguous so once all the vector tables are receieved the
                                                         next data will be the beginning of the application)  */
                                                    else if ( HexRecord.Offset > ((ProgPage.LAddr * HEX_ADD_DVDR) + ((ProgPage.iBuffer*4)/3)))
                                                    {

                                                        /*  write any remaining Program Page buffer to correct memory address */
                                                        /* if lower address is still zero the data coming in must be the vector tables  */
                                                        if ((ProgPage.UAddr == 0)&&( ProgPage.LAddr == 0)                                                            )
                                                        {
                                                            /* read the reset vector from the flash  memory */
                                                            SourceAddr.Val32 = RESET_VECTOR_ADDR;
                                                            TempData1.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);
                                                            SourceAddr.Val32 = RESET_VECTOR_ADDR + 2;
                                                            TempData2.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

                                                            /*  Place the reset vector data (3 bytes) and the following 3 bytes into the buffer at the first 6 locations */
                                                            ProgPage.Buffer[0] =  TempData1.Val[0];
                                                            ProgPage.Buffer[1] =  TempData1.Val[1];
                                                            ProgPage.Buffer[2] =  TempData1.Val[2];
                                                            ProgPage.Buffer[3] =  TempData2.Val[0];
                                                            ProgPage.Buffer[4] =  TempData2.Val[1];
                                                            ProgPage.Buffer[5] =  TempData2.Val[2];
                                                        } /* if lower address is still zero the data coming in must be the vector tables  */

                                                        SourceAddr.Word.HW = ProgPage.UAddr;
                                                        SourceAddr.Word.LW = ProgPage.LAddr;
                                                        WritePM( ProgPage.Buffer, SourceAddr);

                                                        /*  initialise buffer to 0xFF, reset buffer pointer and set lower Program Page address   */
                                                        for ( ProgPage.iBuffer = 0; ProgPage.iBuffer < PM_PAGE_SIZE; ProgPage.iBuffer++)
                                                            ProgPage.Buffer[ProgPage.iBuffer] = 0xFF;

                                                        ProgPage.LAddr = HexRecord.Offset/HEX_ADD_DVDR;
                                                        ProgPage.iBuffer = 0;

                                                        /* place data from HexRecord into Program Page Buffer discarding every fourth byte (zero padding)   */
                                                        for (   iData = 0; iData < HexRecord.RecLength; iData++)
                                                        {
                                                            if ( (iData + 1) % 4)
                                                            {
                                                                ProgPage.Buffer[ProgPage.iBuffer++] = HexRecord.Data[iData];
                                                            }
                                                        }

                                                    }   /* else if HexRecord (byte) address is greater than the (byte) address buffer */

                                                }       /*  correct memory range   */

                                                /*  Assert CTS  */
                                                LATFbits.LATF13 = 1;

                                                /* get next hex record */
                                                DnldStatus = WAIT_PR_RECMARK;

                                            break;  /* case RT_DATA_REC */

                                            case    RT_EOF_REC:
                                                /*  dissassert CTS  */
                                                LATFbits.LATF13 = 0;

                                                /*  write any remaining Program Page buffer to correct memory address */
                                                if ( ProgPage.iBuffer != 0)
                                                {
                                                        /* if lower address is still zero the data coming in must be the vector tables  */
                                                        if ( ProgPage.LAddr == 0)
                                                        {
                                                            /* read the reset vector from the flash  memory */
                                                            SourceAddr.Val32 = RESET_VECTOR_ADDR;
                                                            TempData1.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);
                                                            SourceAddr.Val32 = RESET_VECTOR_ADDR + 2;
                                                            TempData2.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

                                                            /*  Place the reset vector data (3 bytes) and the following 3 bytes into the buffer at the first 6 locations */
                                                            ProgPage.Buffer[0] =  TempData1.Val[0];
                                                            ProgPage.Buffer[1] =  TempData1.Val[1];
                                                            ProgPage.Buffer[2] =  TempData1.Val[2];
                                                            ProgPage.Buffer[3] =  TempData2.Val[0];
                                                            ProgPage.Buffer[4] =  TempData2.Val[1];
                                                            ProgPage.Buffer[5] =  TempData2.Val[2];
                                                        }


                                                        /*  program the memory  */
                                                        SourceAddr.Word.HW = ProgPage.UAddr;
                                                        SourceAddr.Word.LW = ProgPage.LAddr;
                                                        WritePM( ProgPage.Buffer, SourceAddr);
                                                }

                                                /*   buffer address and pointer reset will be reset when next programming activity takes place */

                                                /*  Assert CTS  */
                                                LATFbits.LATF13 = 1;

                                                /* end of file  */
                                                CommandPending = SYSCOM_PROG;
                                                CommandStatus = WAIT_EOX;


                                            break;  /* case RT_EOF_REC */

                                            case    RT_ESA_REC:
                                                DnldStatus = WAIT_PR_RECMARK;
                                            break;  /* case RT_ESA_REC */

                                            case    RT_SSA_REC:
                                                DnldStatus = WAIT_PR_RECMARK;
                                            break;  /* case RT_SSA_REC */

                                            case    RT_ELA_REC:         /* Extended Linear address Record  */
                                                /*  dissassert CTS  */
                                                LATFbits.LATF13 = 0;

                                                /*  write any remaining Program Page buffer to correct memory address */
                                                if (( ProgPage.iBuffer != 0) && (ProgPage.UAddr == 0))
                                                {
                                                    SourceAddr.Word.HW = ProgPage.UAddr;
                                                    SourceAddr.Word.LW = ProgPage.LAddr;
                                                    WritePM( ProgPage.Buffer, SourceAddr);
                                                }

                                                /*  reset lower Program Page address and buffer pointer */
                                                for ( ProgPage.iBuffer = 0; ProgPage.iBuffer < PM_PAGE_SIZE; ProgPage.iBuffer++)
                                                    ProgPage.Buffer[ProgPage.iBuffer] = 0xFF;

                                                ProgPage.LAddr = 0;
                                                ProgPage.iBuffer = 0;

                                                /* set the upper address using the 2 bytes of data held */
                                                ProgPage.UAddr = (( HexRecord.Data[0] << 8 ) + HexRecord.Data[1])/ HEX_ADD_DVDR ;

                                                DnldStatus = WAIT_PR_RECMARK;

                                                /*  Assert CTS  */
                                                LATFbits.LATF13 = 1;

                                            break;  /* case RT_ELA_REC */

                                            case    RT_SLA_REC:
                                                DnldStatus = WAIT_PR_RECMARK;
                                            break;  /* case RT_SLA_REC */

                                        }   /* switch (HexRecord.Type)   */
                                    }   /* if (HexRecord.ChkSum == TempData) */
                                }   /*  if (HexRecord.ChkSum == InData) */

                            break;  /* case WAIT_PR_CHKSUM */

                        }   /* if ((InData == COLON)||(isxdigit(Indata)) */

                    }   /* switch(DnldStatus) */

                break;  /* case WAIT_DNLD */

                case WAIT_EOX:   /* Waiting for final byte */
                {
                    if ( InData == EOX_BYTE )
                    {
                        switch(CommandPending)
                        {
                            case SYSCOM_ERASE:   /*  SYSCOM_ERASE command completed  */
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;

                                /* Erase program memory */
                                EraseProgramMemory();

                                SendResponse( RESPONSE_ACK ); /* send ACK  */

                            break;

                            case SYSCOM_PROG:   /*  SYSCOM_PROG command completed */
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                                SendResponse( RESPONSE_ACK ); /* send ACK  */

                                /* Wait for command to be sent */
                                while (!SERIALOutBufferIsEmpty())
                                {
                                    /* Service serial routines */
                                    if(IFS1bits.U2TXIF)
                                    {
                                        U2TXInterrupt();
                                    }
                                }

                                /* Wait for last byte to be tansmitted */
                                while(!IFS1bits.U2TXIF);

                                /* Wait for TX shift register to empty */
                                for (Delay.Val32 = 0; Delay.Val32 < TX_DELAY_COUNT; Delay.Val32++)
                                {
                                    Nop();
                                }

                                CloseUART2();

                                /* Clear Bootload Word */
                                ClearBootloadVector();

                                /* Reset device to run application */
                                GoResetDevice();

                                /* Point of no return ! */
                            break;

                            case SYSCOM_VERSION_READ:
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                                SendResponse( RESPONSE_VER_NO ); /* send Version Number  */
                            break;

                            case SYSCOM_MODE_READ:
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                                SendResponse( RESPONSE_MODE ); /* send Mode  */
                            break;

                            case SYSCOM_BOOTLOAD:
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                                SendResponse( RESPONSE_ACK ); /* send ACK (already in Bootload mode !) */
                            break;

                            default:    /* invalid command pending, start wait for command again */
                            case SYSCOM_NONE:   /*  Unrecognised command completed */
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                                SendResponse( RESPONSE_NACK ); /* send NACK  */
                            break;

                        } /*    switch(CommandPending)  */
                    }
                }   /* WAIT_EOX */

                default:    /* ignore any other input */

                break;
            }   /* switch(CommandStatus) */

        }  /*  if (!SERIALInBufferEmpty()) */

        /* Service serial routines */
        if(IFS1bits.U2TXIF)
        {
            U2TXInterrupt();
        }

        //if(IFS1bits.U2RXIF)
        if (U2STAbits.URXDA)
        {
            U2RXInterrupt();
        }

    } //end while

#ifdef NOTYET
    while (0)
    {
        char Command;
        GetChar(&Command);

        switch(Command)
        {
            case COMMAND_READ_PM:               /*tested*/
            {
                uReg32 SourceAddr;

                GetChar(&(SourceAddr.Val[0]));
                GetChar(&(SourceAddr.Val[1]));
                GetChar(&(SourceAddr.Val[2]));
                SourceAddr.Val[3]=0;

                ReadPM(Buffer, SourceAddr);

                WriteBuffer(Buffer, PM_ROW_SIZE*3);

                break;
            }

            case COMMAND_WRITE_PM:              /* tested */
            {
                uReg32 SourceAddr;
                int    Size;

                GetChar(&(SourceAddr.Val[0]));
                GetChar(&(SourceAddr.Val[1]));
                GetChar(&(SourceAddr.Val[2]));
                SourceAddr.Val[3]=0;

                for(Size = 0; Size < PM_ROW_SIZE*3; Size++)
                {
                GetChar(& ));
                }

                Erase(SourceAddr.Word.HW,SourceAddr.Word.LW,PM_ROW_ERASE);

                WritePM(Buffer, SourceAddr);        /*program page */

                PutChar(COMMAND_ACK);               /*Send Acknowledgement */

                break;
            }

            case COMMAND_READ_ID:
            {
                uReg32 SourceAddr;
                uReg32 Temp;

                SourceAddr.Val32 = 0xFF0000;

                Temp.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

                WriteBuffer(&(Temp.Val[0]), 4);

                SourceAddr.Val32 = 0xFF0002;

                Temp.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

                WriteBuffer(&(Temp.Val[0]), 4);

                break;
            }

            case COMMAND_WRITE_CM:
            {
                int    Size;

                for(Size = 0; Size < CM_ROW_SIZE*3;)
                {
                    GetChar(&(Buffer[Size++]));
                    GetChar(&(Buffer[Size++]));
                    GetChar(&(Buffer[Size++]));

                    PutChar(COMMAND_ACK);               /*Send Acknowledgement */
                }


                break;
            }
                case COMMAND_RESET:
            {
                uReg32 SourceAddr;
                int    Size;
                uReg32 Temp;


                for(Size = 0, SourceAddr.Val32 = 0xF80000; Size < CM_ROW_SIZE*3;
                                                                Size +=3, SourceAddr.Val32 += 2)
                {
                    if(Buffer[Size] == 0)
                    {
                        Temp.Val[0]=Buffer[Size+1];
                        Temp.Val[1]=Buffer[Size+2];

                        WriteLatch( SourceAddr.Word.HW,
                                        SourceAddr.Word.LW,
                                        Temp.Word.HW,
                                        Temp.Word.LW);

                        WriteMem(CONFIG_WORD_WRITE);
                    }
                }


                GoResetDevice();

                break;
            }

            case COMMAND_NACK:
            {
                GoResetDevice();

                break;
            }


            default:
                PutChar(COMMAND_NACK);
                break;
        }

    }
#endif
}

/*****************************************************************************
*
* Name:    SendResponse
*
* Description: Sends response to SYSEX message
*
*
*****************************************************************************/
void SendResponse(tResponseType ResponseType )
{
    char * VersionString = FW_NUMBER;

    /* send Status byte and manufacturers ID  */
    SERIALWriteByte (STATUS_BYTE);
    SERIALWriteByte (ID1);
    SERIALWriteByte (ID2);
    SERIALWriteByte (ID3);

    switch(ResponseType)
    {
        case RESPONSE_VER_NO:
            SERIALWriteByte (CODE_VER_NO);
            SERIALWriteByte (*VersionString++);
            SERIALWriteByte (*VersionString++);
            SERIALWriteByte (*VersionString++);
            SERIALWriteByte (*VersionString);
        break;

        case RESPONSE_MODE:
            SERIALWriteByte (CODE_MODE);
            SERIALWriteByte ((byte)SYSSTATUS_BOOTMODE);
        break;

        case RESPONSE_ACK:
            SERIALWriteByte (CODE_ACK);
        break;

        case RESPONSE_NACK:
        default:
            SERIALWriteByte (CODE_NACK);
        break;
    }

    /* End of message  */
    SERIALWriteByte (EOX_BYTE);
}

/******************************************************************************/
void GetChar(char * ptrChar)
{
    while(1)
    {
        /* if timer expired, signal to application to jump to user code */
        if(IFS0bits.T3IF == 1)
        {
            * ptrChar = COMMAND_NACK;
            break;
        }
        /* check for receive errors */
        if(U1STAbits.FERR == 1)
        {
            continue;
        }

        /* must clear the overrun error to keep uart receiving */
        if(U1STAbits.OERR == 1)
        {
            U1STAbits.OERR = 0;
            continue;
        }

        /* get the data */
        if(U1STAbits.URXDA == 1)
        {
            T2CONbits.TON=0; /* Disable timer countdown */
            * ptrChar = U1RXREG;
            break;
        }
    }
}


/******************************************************************************/
void ReadPM(char * ptrData, uReg32 SourceAddr)
{
    int    Size;
    uReg32 Temp;

    for(Size = 0; Size < PM_ROW_SIZE; Size++)
    {
        Temp.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

        ptrData[0] = Temp.Val[2];;
        ptrData[1] = Temp.Val[1];;
        ptrData[2] = Temp.Val[0];;

        ptrData = ptrData + 3;

        SourceAddr.Val32 = SourceAddr.Val32 + 2;
    }
}
/******************************************************************************/

void WriteBuffer(char * ptrData, int Size)
{
    int DataCount;

    for(DataCount = 0; DataCount < Size; DataCount++)
    {
        PutChar(ptrData[DataCount]);
    }
}
/******************************************************************************/
void PutChar(char Char)
{
    while(!U1STAbits.TRMT);

    U1TXREG = Char;
}
/******************************************************************************/

void WritePM(char * ptrData, uReg32 SourceAddr)
{
    int    Size,Size1;
    uReg32 Temp;
    uReg32 TempAddr;
    uReg32 TempData;

    for(Size = 0,Size1=0; Size < PM_ROW_SIZE; Size++)
    {

        Temp.Val[0]=ptrData[Size1+0];
        Temp.Val[1]=ptrData[Size1+1];
        Temp.Val[2]=ptrData[Size1+2];
        Temp.Val[3]=0;
        Size1+=3;

       WriteLatch(SourceAddr.Word.HW, SourceAddr.Word.LW,Temp.Word.HW,Temp.Word.LW);

        /* Device ID errata workaround: Save data at any address that has LSB 0x18 */
        if((SourceAddr.Val32 & 0x0000001F) == 0x18)
        {
            TempAddr.Val32 = SourceAddr.Val32;
            TempData.Val32 = Temp.Val32;
        }

        if((Size !=0) && (((Size + 1) % 64) == 0))
        {
            /* Device ID errata workaround: Reload data at address with LSB of 0x18 */
          WriteLatch(TempAddr.Word.HW, TempAddr.Word.LW,TempData.Word.HW,TempData.Word.LW);

            WriteMem(PM_ROW_WRITE);
        }

        SourceAddr.Val32 = SourceAddr.Val32 + 2;
    }


}


/******************************************************************************/

void GoResetDevice(void)
{
    ResetDevice();
}

/******************************************************************************/

/**
**  Clear BOOTLOAD_WORD_ADDR to zero to enable application !!
*/

void ClearBootloadVector(void)
{
    DWORD TempAddr;
    DWORD TempData;

    TempAddr._dword = BOOTLOAD_WORD_ADDR;   /* Location of redirected reset vector */
    TempData._dword = 0x0000;   /* Data to be written */

    //InterruptDisable ();

    WriteLatch(TempAddr.word1, TempAddr.word0,TempData.word1,TempData.word0);

    WriteMem(PM_WORD_WRITE);

    InterruptEnable ();
}

/*
** Enable interrupts to CPU
**
*/
static int old_ipl = 0;

void InterruptEnable (void)
{
    RESTORE_CPU_IPL(old_ipl);
}

/*
** Disable interrupts to CPU
**
*/
void InterruptDisable (void)
{
    SET_AND_SAVE_CPU_IPL(old_ipl, 7);
}

/******************************************************************************
**
**  Erase interrupt vectors and area where Midi Keyboard Application is resident
**  but reprogram reset vector
*/

void EraseProgramMemory(void)
{
    uReg32 SourceAddr;
    uReg32 TempData1, TempData2;

    /* read reset vector */
    SourceAddr.Val32 = RESET_VECTOR_ADDR;
    TempData1.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);
    SourceAddr.Val32 = RESET_VECTOR_ADDR + 2;
    TempData2.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);

    /* erase memory up to bootload start  */
    SourceAddr.Val32 = RESET_VECTOR_ADDR;
    Erase(SourceAddr.Word.HW,SourceAddr.Word.LW,PM_ROW_ERASE);

    SourceAddr.Val32 += PM_ROW_SIZE;
    Erase(SourceAddr.Word.HW,SourceAddr.Word.LW,PM_ROW_ERASE);


    /*  clear buffer  */
    for ( ProgPage.iBuffer = 0; ProgPage.iBuffer < PM_PAGE_SIZE; ProgPage.iBuffer++)
        ProgPage.Buffer[ProgPage.iBuffer] = 0xFF;

    /*  Place the reset vector data (3 bytes) and the following 3 bytes into the buffer at the first 6 locations */
    ProgPage.Buffer[0] =  TempData1.Val[0];
    ProgPage.Buffer[1] =  TempData1.Val[1];
    ProgPage.Buffer[2] =  TempData1.Val[2];
    ProgPage.Buffer[3] =  TempData2.Val[0];
    ProgPage.Buffer[4] =  TempData2.Val[1];
    ProgPage.Buffer[5] =  TempData2.Val[2];

    /* set the write address  */
    SourceAddr.Val32 = RESET_VECTOR_ADDR;

    /* complete the write  */
    WritePM( ProgPage.Buffer, SourceAddr);

    /* erase application memory  */
    for(SourceAddr.Val32 = APP_START_ADDR; SourceAddr.Val32 < EEPROM_START_ADDR; (SourceAddr.Val32 += PM_ROW_SIZE) )
    {
        Erase(SourceAddr.Word.HW,SourceAddr.Word.LW,PM_ROW_ERASE);
    }
}

/******************************************************************************
**
**  A utility to convert an ascii hex numeral to it's numeric value
*/


byte ConvertASCII ( byte ch)
{
    if ((ch >= '0') && (ch <= '9'))
        return (ch - '0');
    else if ((ch >= 'A') && (ch <= 'F'))
        return (ch - 'A' + 10);
    else if ((ch >= 'a') && (ch <= 'f'))
        return (ch - 'a' + 10); //
    else
        return -1;
}
/* EOF */
