/**********************************************************************
* Filename          : Ad.c $workfile: $
*
* Module            :
*
* Author            : Johh Thorne
* Description       : This provides the ADC Access
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   0.0     22Jan2008   -               First Issue
*
*   $Log: ad.c,v $
*   Revision 1.1  2009/09/22 15:55:12  rt
*   Initial commit to CVS.
*   Originally version 00.00.20
*   Committed on the Free edition of March Hare Software CVSNT Server.
*   Upgrade to CVS Suite for more features and support:
*   http://march-hare.com/cvsnt/
*
*
*
* Copyright         : Cambridge Micro Engineering, Jan 2008
*
***********************************************************************/

#include <p33fxxxx.h>
#include <stdio.h>

#include "ad.h"

/*==========================================================================*/
/* Type definitions and module specific defines                             */
/*==========================================================================*/
#define NUM_ADC_RESULTS 10

#define ADCMAX  1023    /* 10bit ADC */
#define ADCMID  (ADCMAX/2)
#define ADCMIN  0

#define tADCResult unsigned int

/*
** Nominal frequency of the system clock 
*/
#define INTRC_CLOCK         (7370000UL)     /* 7.37MHz (internal oscillator) */
#define OSCILLATOR_CLOCK    (79227500UL)    /* 79.2275MHz */
#define MASTER_CLOCK        (OSCILLATOR_CLOCK / 2UL)    /* 39.61MHz */
#define TICKS_PER_MS        (MASTER_CLOCK/1000L)


/*==========================================================================*/
/* Forward declarations of functions in this module                         */
/*==========================================================================*/


/*==========================================================================*/
/* Variables                                                                */
/*==========================================================================*/

volatile tADCResult ADResultArray[NUM_ADC_RESULTS] = {
ADC_INVALID_READING, 
ADC_INVALID_READING, 
ADC_INVALID_READING, 
ADC_INVALID_READING,
ADC_INVALID_READING,
ADC_INVALID_READING,
ADC_INVALID_READING,
ADC_INVALID_READING,
ADC_INVALID_READING,
ADC_INVALID_READING
};

volatile tADCResult ADResult = ADC_INVALID_READING;

/*==========================================================================*/




/*==========================================================================*/
/* IMPLEMENTATION                                                           */
/*==========================================================================*/

/*****************************************************************************
*
* Name:    AdInitialise
*
* Description:  ADC Initialisation
*
*               Called to initialise ADC for battery monitor
*               Note: ADCs used for motor current protection are initialised
*               in motor.c, not here.
*
*               VBAT uses AN4
*               VUSB uses AN5
*
*****************************************************************************/

void AdInitialise (void)
{
        //ADCON1 Register
        //Set up A/D for Automatic Sampling
        //Use internal counter (SAMC) to provide sampling time
        //Set up A/D conversrion results to be read in 1.15 fractional
        //number format.
        //Set up Sequential sampling for multiple S/H amplifiers
        //All other bits to their default state
		AD1CON1bits.AD12B  = 1;	/* 12-bit, 1-channel ADC operation */
        AD1CON1bits.FORM   = 0;	/* 12bit unsigned Integer */
        AD1CON1bits.SSRC   = 7;	/* the conversion trigger is under A/D clock control. */
        AD1CON1bits.ASAM   = 0;	/* Sampling begins when SAMP bit is set  */
        AD1CON1bits.SIMSAM = 0;	/* Samples multiple channels individually in sequence */

        //ADCON2 Register
        //Set up A/D for interrupting after single sample filed in the buffer
        //Set up to sample on 1 S/H amplifiers - CH0, CH1, CH3 and CH4
        //All other bits to their default state
        AD1CON2bits.SMPI = 0;	/* Generates interrupt after completion of every sample/conversion operation */
        AD1CON2bits.CHPS = 0;	/* Converts CH0 */
		AD1CON2bits.VCFG = 0; 	/* Internal AVDD and AVSS		  */
		AD1CON2bits.BUFM = 0;	/* Starts buffer filling at address 0x0 */

        //ADCON3 Register
        //We would like to set up the slowest possible sampling rate
		AD1CON3bits.ADRC = 1; 	/* ADC Clock derived from internal RC */
        AD1CON3bits.SAMC = 31;	/* Auto Sample Time bits */
        AD1CON3bits.ADCS = 255;	/* See above */

        //ADCHS Register
        //Set up A/D Channel Select Register to convert CH0
		//CH1, CH2, CH3 negative input is VREF and
		//CH1 positive input is AN0, CH2 positive input is AN1, CH3 positive input is AN2
		AD1CHS123 = 0;

		// And CH0 positive is on AN0
		AD1CHS0   = 0;

        //ADCSSL Register
        //Channel Scanning on AN0, AN1, AN2 and AN3
        AD1CSSL = 0x0000;

        //ADPCFG Register
        //Set up channels AN0 as analog input and configure rest as digital
        AD1PCFGL = 0xFFFE;

        //Clear the A/D interrupt flag bit
        IFS0bits.AD1IF = 0;

        //Set the A/D interrupt enable bit
        IEC0bits.AD1IE = 1;

        //Turn on the A/D converter
        //This must be done after configuring other registers
        AD1CON1bits.ADON = 1;

		AD1CON1bits.SAMP = 1;	/* Start sampling ! */


        InitTMR3( );

}


/*********************************************************************
  Function:        void InitTMR3(void)

  PreCondition:    None.

  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        Initialization of timer 3 as the start for sample
                   conversion every second.

  Note:            None.
********************************************************************/

void InitTMR3( void )
{
    T3CONbits.TON   = 0;    /* Turn Timer off to initialise */
    T3CONbits.TSIDL = 0;    /* Continue in IDLE mode */
    T3CONbits.TGATE = 0;    /* Gate function diabled */
    T3CONbits.TCKPS = 0;    /* Pre-scaler 1:1 */
    T3CONbits.TCS   = 0;    /* Internal Tcy clock source */

    TMR3 = 0;
    PR3  = TICKS_PER_MS;

    IPC2bits.T3IP = 0x01; // Set Timer 1 Interrupt Priority Level
    IFS0bits.T3IF = 0; // Clear Timer 1 Interrupt Flag
    IEC0bits.T3IE = 1; // Enable Timer1 interrupt

    T3CONbits.TON   = 1;    /* Turn Timer on */
}

/****** START OF INTERRUPT SERVICE ROUTINES *********/


/*********************************************************************
  Function:        void _ADCInterrupt(void)

  PreCondition:    None.
 
  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        Interrupt service routine for ADC capture channels.
                   Routine also calculates average over last 
                   NUM_ADC_RESULTS results.

  Note:            None.
********************************************************************/

void __attribute__((__interrupt__, auto_psv)) _ADC1Interrupt(void)
{
    static uint8_t ADCResultIndex = 0;

    /* Stop sampling */
    AD1CON1bits.SAMP   = 0;

	ADResultArray[ADCResultIndex++]= ADC1BUF0;

    /* just overwrite oldest result when buffer full */
    if (ADCResultIndex >= NUM_ADC_RESULTS)
    {
        ADCResultIndex = 0;
    }

    //Clear the A/D Interrupt flag bit or else the CPU will
    //keep vectoring back to the ISR
    IFS0bits.AD1IF = 0;
      
}


void __attribute__((__interrupt__, auto_psv)) _T3Interrupt(void)
{
    static unsigned MsTicks = 1000;

    /* Clear interrupt flag */
    IFS0bits.T3IF = 0;

    if (--MsTicks)
    {
        /* Start sampling ! */ 
        AD1CON1bits.SAMP = 1;	

        MsTicks = 100;
    }
}

/************************************************
 End of File
*************************************************/
