/*****************************************************************************
* Filename          : buffer.c
*
* Module            : 
*
* Author            : V A Tarrant
* Description       : This module holds buffer to hold data until it  
*                     has been processed
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   1.0     10May2007   Val Tarrant     First Issue
*
*
* Copyright         : Cambridge Micro Engineering, May 2007
*
******************************************************************************/

/*****************************************************************************
* Library Includes
*****************************************************************************/
#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#include <libpic30.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#include <libpic24.h>
#endif

#include    <string.h>

/*****************************************************************************
* Project Includes
*****************************************************************************/
#include    "typedefs.h"
#include    "midi.h"
#include	"buffer.h"


/******************************************************************************
*   Local Constants
******************************************************************************/



/*****************************************************************************
*   Global Variables
******************************************************************************/

/* 	Buffer to store data elements until they can be processes by the background task. 
    This may be accessed under interrupt - hence volatile  */
volatile tDataElement Buffer[MAX_BUFFER_SIZE]; /* Circular buffer for data */

volatile tBuffer bufFrame;

#ifdef REMOVED
/* The index to the next free element in buffer */
volatile U8  iBUFFERIn= 0;

/* The index to the next value to be taken from Buffer */
volatile U8    iBUFFEROut= 0; 

/* Used to indicate if there are any data in buffer to be processed */
volatile BOOLEAN    BUFFEREmptyStatus = TRUE;

/* used todetermine if buffer is full */
volatile BOOLEAN    BUFFERFullStatus = FALSE;
#endif



/*****************************************************************************
*   Local Function Prototypes
*****************************************************************************/



/*****************************************************************************
*   Exported functions
*****************************************************************************/
/*****************************************************************************
*
* Name:         BUFFEREmpty
*
* Description:  Returns true if buffer is empty else false 
*          
*****************************************************************************/
BOOLEAN BUFFEREmpty(tBuffer * buf)
{
    return buf->BUFFEREmptyStatus;
}

/*****************************************************************************
*
* Name:         BUFFERFull
*
* Description:  Returns true if buffer is full else false 
*          
*****************************************************************************/

BOOLEAN BUFFERFull(tBuffer *buf)
{
    return buf->BUFFERFullStatus;
}

/*****************************************************************************
*
* Name:         BUFFERRead
*
* Description:  Fill data structure from foreground task with next 
*               out element in buffer 
*                
*          
*****************************************************************************/
BOOLEAN BUFFERRead( tBuffer *buf, tDataElement *DataElement)
{
    word old_ipl;                                               

    if(buf->BUFFEREmptyStatus)
    {
        return FALSE;   /* Nodata to collect */
    }

	/* Retrieve DataElement from buffer */
	memcpy(DataElement, &buf->Buffer[buf->iBUFFEROut], sizeof(DataElement));

	/* Mark buffer as not full */
    buf->BUFFERFullStatus = FALSE;

	/* There is data so update Buffer */
	/* give cpu high priority therefore interrupts */
	SET_AND_SAVE_CPU_IPL(old_ipl, 7);  

	/* increment out pointer and wrap  */
	buf->iBUFFEROut = ((buf->iBUFFEROut + 1)  & (MAX_BUFFER_SIZE-1));

	/* if out pointer == to in pointer buffer empty */
	if (buf->iBUFFEROut == buf->iBUFFERIn)
		buf->BUFFEREmptyStatus = TRUE;

    /* restore CPU priority to previous value allow interrupts */
	RESTORE_CPU_IPL(old_ipl);

    return TRUE;
}   /* End BUFFERRead */

/*****************************************************************************
*
* Name:         BUFFERStore
*
* Description:  Saves tDataElement contents to next element in buffer, 
*               during foregroud task.
*          
* Return:       TRUE if data stored
*               FALSE if data overrun (storage was full)
*
*****************************************************************************/
BOOLEAN BUFFERWrite( tBuffer *buf, tDataElement *DataElement)
{
    word old_ipl;    

    if (buf->BUFFERFullStatus)
    {
        /* If buffer is full then flag overrun */
        return FALSE;
    }

    /* Retrive Data element to store */
	memcpy(&buf->Buffer[buf->iBUFFERIn], DataElement, sizeof (DataElement));

	/* give cpu high priority therefore interrupts */
	SET_AND_SAVE_CPU_IPL(old_ipl, 7);  

    /* Increment pointer and wrap if past the end of the buffer */
    buf->iBUFFERIn = ((buf->iBUFFERIn+1)  & (MAX_BUFFER_SIZE-1));

	/* buffer not empty */
    buf->BUFFEREmptyStatus = FALSE;

	/* if in pointer == to out pointer buffer full */
	if (buf->iBUFFERIn == buf->iBUFFEROut)
		buf->BUFFERFullStatus = TRUE;
 
   /* restore CPU priority to previous value allow interrupts */
	RESTORE_CPU_IPL(old_ipl);

    return TRUE;

}   /* End BUFFERStore */

/*****************************************************************************
*
* Name:         BUFFERInit
*
* Description:  Clears the BUFFER Pointers to nulls
*          
*          
*
*****************************************************************************/
void    BUFFERInit  ( void )
{
    word old_ipl;

	/* Give cpu high priority therefore interrupts */
	SET_AND_SAVE_CPU_IPL(old_ipl, 7);  

    bufFrame.iBUFFERIn= 0;
    bufFrame.iBUFFEROut= 0;
    bufFrame.BUFFERFullStatus = FALSE;
    bufFrame.BUFFEREmptyStatus = TRUE;
    bufFrame.Buffer = (tDataElement *) Buffer;

    /* Restore CPU priority to previous value allow interrupts */
	RESTORE_CPU_IPL(old_ipl);
}

/* EOF */
