/*****************************************************************************
* Filename          : keybrd.c
*
* Module            : 
*
* Author            : Val Tarrant/John Thorne
* Description       : This module holds the functions for the 
*                     HT-MINI MIDI Keyboard scanning
*	
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   1.0     30Jan2008   Val Tarrant     First Issue
*
*
* Copyright         : Cambridge Micro Engineering, Jan 2008
*
******************************************************************************/

/*****************************************************************************
* Library Includes
*****************************************************************************/

#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#include <libpic30.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#include <libpic24.h>
#endif

#include <stdio.h>
#include <string.h>

/*****************************************************************************
* Project Includes
*****************************************************************************/
#include    "typedefs.h"
#include	"keybrd.h"
#include	"midi.h"
#include	"io_cfg.h"
/******************************************************************************
*   Local Constants
******************************************************************************/
typedef struct sSwitchLocation
{
	unsigned char Col;
	unsigned char Row;
} tSwitchLocation;

typedef struct sKeyLocation
{
    tSwitchLocation Sw2;
    tSwitchLocation Sw1;
} tKeylocation;

/* array of midi codes set out in "rows" */ 
const char MIDICode[NUM_KEYS] =  {
81, 78, 82, 79, 83, 80, 84, 81, 78, 82, 79, 83, 80, 84,
74, 71, 75, 72, 76, 73, 77, 74, 71, 75, 72, 76, 73, 77,
67, 64, 68, 65, 69, 66, 70, 67, 64, 68, 65, 69, 66, 70,
60, 57, 61, 58, 62, 59, 63, 60, 57, 61, 58, 62, 59, 63,
53, 50, 54, 51, 55, 52, 56, 53, 50, 54, 51, 55, 52, 56,
46, 43, 47, 44, 48, 45, 49, 46, 43, 47, 44, 48, 45, 49,
39, 36, 40, 37, 41, 38, 42, 39, 36, 40, 37, 41, 38, 42
};

/* array of midi codes set out in "rows" */ 
const char MIDITestCode[NUM_KEYS] =  {
 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13,
14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27,
28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41,
42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83,
84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97
};

/* array of midi codes set out in "rows" */ 
const char MIDIAlt1Code[NUM_KEYS] =  {
 1,  8, 15, 22, 29, 36, 43, 50, 57, 64, 71, 78, 85, 92,
 2,  9, 16, 23, 30, 37, 44, 51, 58, 65, 72, 79, 86, 93,
 3, 10, 17, 24, 31, 38, 45, 52, 59, 66, 73, 80, 87, 94,
 4, 11, 18, 25, 32, 39, 46, 53, 60, 67, 74, 81, 88, 95,
 5, 12, 19, 26, 33, 40, 47, 54, 61, 68, 75, 82, 89, 96,
 6, 13, 20, 27, 34, 41, 48, 55, 62, 69, 76, 83, 90, 97,
 7, 14, 21, 28, 35, 42, 49, 56, 63, 70, 77, 84, 91, 98
};

/*****************************************************************************
*   Local Variables
******************************************************************************/

/* This is a pointer to the keyboardscan that needs to be analysed           */
/* The pointer is null until set by the ADC software after a complete        */
/* keyboard scan has been performed                                          */
tRowScan * NextKeyboardScan = NULL;

tRowScan KeyboardScan[SCANS_PER_ROW][NUM_ROWS];     /* Holds the instantaneous state of switches */
tRowScan LastKeyboardScan[NUM_ROWS];                /* Holds the last instantaneous state of switches */
tRowScan SwitchStatus[NUM_ROWS];	                 /* Holds the transision state of switches */

tKeyStatus KeyStatus[NUM_KEYS];			/* Holds the key status */

char const * MIDIKeyCode;   /* pointer to correct table of codes for Application/test status  */

tKeyboardMode KeyboardMode;

#define MAX_TRANSPOSITON    3   /* in octaves */
#define SEMITONES_PER_OCTAVE     12

/* Each transposition switch is checked at 10 ms intervals
but only one switch is checked at a time so needs to be checked every 5ms */

#define TCOUNT_RELOAD  4
#define DEBOUNCE_COUNT 4
#define KEYOFF_TIMEOUT 1000 

enum
{
    SWITCH_WAIT,
    SWITCH_WAIT_UP,
    SWITCH_WAIT_DOWN,
    SWITCH_DEBOUNCE,
    SWITCH_NUM_STATES
};

static byte TCount = TCOUNT_RELOAD;           

static char OctaveTransposition = 0;   /* holds octave movement */
static char Transposition = 0;     /* Holds transposition value +ve value semitones up
                                                         0 no transposition
                                                         -ve value semitones down */

static tTSwitch Key = SWITCH_T_UP;
static byte switchState[NUM_T_SWITCHES] = { SWITCH_WAIT, SWITCH_WAIT };
static byte switchDebounceTimeout[NUM_T_SWITCHES] = { 0, 0};
static byte switchLast[NUM_T_SWITCHES] = { SWITCH_RELEASED, SWITCH_RELEASED };
static byte switchNew[NUM_T_SWITCHES];

//static byte KeyOnCount = 0;
//static BOOLEAN MIDIAllOffSent = FALSE;
//static word KeyOffTimeout = KEYOFF_TIMEOUT;

/*****************************************************************************
*   Global Variables
******************************************************************************/


/*****************************************************************************
*   Local Function Prototypes
*****************************************************************************/


/*****************************************************************************
*
* Name:    KEYBRDInit
*          
* Description:  Initialise MIDIKeyCode to point to the correct keymap
*
*
*****************************************************************************/
void KEYBRDInit (tKeyboardMode Mode)
{
	//memset(KeyStatus, 0, sizeof(KeyStatus));
	//memset(KeyboardScan, 0 ,sizeof(KeyboardScan));
	//memset(NextKeyboardScan, 0 ,sizeof(KeyboardScan));
     
	switch (Mode)
	{
		default:
		case KEYBOARD_MODE_MIDI:
	        MIDIKeyCode = &MIDICode[0];
			KeyboardMode = KEYBOARD_MODE_MIDI;
			break;

		case KEYBOARD_MODE_ALT:
	    	MIDIKeyCode = &MIDIAlt1Code[0];
			KeyboardMode = KEYBOARD_MODE_ALT;
			break;

		case KEYBOARD_MODE_TEST:
		    MIDIKeyCode = &MIDITestCode[0];
			KeyboardMode = KEYBOARD_MODE_TEST;
			break;
	}
    Transposition = OctaveTransposition = 0;
}

/*****************************************************************************
*
* Name:    KEYBRDStateMachine
*
* Description: Keyboard State Machine control: This is called every Ms when a 
*              complete A2D scan has been completed for all switches associated 
*              with pitch keys.
*              The transposition switches are checked every TCOUNT_RELOAD times 
*              scanned and debounced after DEBOUNCE_COUNT times
*
*****************************************************************************/
/* Sw1 col Row  Sw2 Col Row  */
const tKeylocation KeyLoc[NUM_KEYS]  = 
{
{{0,  0},{0,  1}}, {{1,  0},{1,  1}}, {{2,  0},{2,  1}}, {{3,  0},{3,  1}}, {{4,  0},{4,  1}}, {{5,  0},{5,  1}}, {{6,  0},{6,  1}},
{{7,  0},{7,  1}}, {{8,  0},{8,  1}}, {{9,  0},{9,  1}}, {{10, 0},{10, 1}}, {{11, 0},{11, 1}}, {{12, 0},{12, 1}}, {{13, 0},{13, 1}},
{{0,  2},{0,  3}}, {{1,  2},{1,  3}}, {{2,  2},{2,  3}}, {{3,  2},{3,  3}}, {{4,  2},{4,  3}}, {{5,  2},{5,  3}}, {{6,  2},{6,  3}},
{{7,  2},{7,  3}}, {{8,  2},{8,  3}}, {{9,  2},{9,  3}}, {{10, 2},{10, 3}}, {{11, 2},{11, 3}}, {{12, 2},{12, 3}}, {{13, 2},{13, 3}},
{{0,  4},{0,  5}}, {{1,  4},{1,  5}}, {{2,  4},{2,  5}}, {{3,  4},{3,  5}}, {{4,  4},{4,  5}}, {{5,  4},{5,  5}}, {{6,  4},{6,  5}},
{{7,  4},{7,  5}}, {{8,  4},{8,  5}}, {{9,  4},{9,  5}}, {{10, 4},{10, 5}}, {{11, 4},{11, 5}}, {{12, 4},{12, 5}}, {{13, 4},{13, 5}},
{{0,  6},{0,  7}}, {{1,  6},{1,  7}}, {{2,  6},{2,  7}}, {{3,  6},{3,  7}}, {{4,  6},{4,  7}}, {{5,  6},{5,  7}}, {{6,  6},{6,  7}},
{{7,  6},{7,  7}}, {{8,  6},{8,  7}}, {{9,  6},{9,  7}}, {{10, 6},{10, 7}}, {{11, 6},{11, 7}}, {{12, 6},{12, 7}}, {{13, 6},{13, 7}},
{{0,  8},{0,  9}}, {{1,  8},{1,  9}}, {{2,  8},{2,  9}}, {{3,  8},{3,  9}}, {{4,  8},{4,  9}}, {{5,  8},{5,  9}}, {{6,  8},{6,  9}},
{{7,  8},{7,  9}}, {{8,  8},{8,  9}}, {{9,  8},{9,  9}}, {{10, 8},{10, 9}}, {{11, 8},{11, 9}}, {{12, 8},{12, 9}}, {{13, 8},{13, 9}},
{{0, 10},{0, 11}}, {{1, 10},{1, 11}}, {{2, 10},{2, 11}}, {{3, 10},{3, 11}}, {{4, 10},{4, 11}}, {{5, 10},{5, 11}}, {{6, 10},{6, 11}},
{{7, 10},{7, 11}}, {{8, 10},{8, 11}}, {{9, 10},{9, 11}}, {{10,10},{10,11}}, {{11,10},{11,11}}, {{12,10},{12,11}}, {{13,10},{13,11}},
{{0, 12},{0, 13}}, {{1, 12},{1, 13}}, {{2, 12},{2, 13}}, {{3, 12},{3, 13}}, {{4, 12},{4, 13}}, {{5, 12},{5, 13}}, {{6, 12},{6, 13}},
{{7, 12},{7, 13}}, {{8, 12},{8, 13}}, {{9, 12},{9, 13}}, {{10,12},{10,13}}, {{11,12},{11,13}}, {{12,12},{12,13}}, {{13,12},{13,13}},

};

void KEYBRDStateMachine(void)
{
	unsigned char i, change, j, k, l, m;

    /* Pitch Switches */

	/* Firstly deduce all switch changes NOT state */
	for (i = 0; i < NUM_ROWS; i++)
	{
		SwitchStatus[i]._ColumnData = LastKeyboardScan[i]._ColumnData ^ NextKeyboardScan[i]._ColumnData;
	}

	/* Remember current switch states */ 
	memcpy(LastKeyboardScan, NextKeyboardScan, sizeof(LastKeyboardScan));

	/* Test each key for state changes depending on its associated switch pair */
	for (i = 0; i < NUM_KEYS; i++)
	{
		/* Deduce if there has been a change of state in either switch assoicated with this key
         *
		 * Possible change configurations are:
         *
         *        00 - No change, so ignore
         *        01 - Switch 1 (inner) has changed
         *        10 - Switch 2 (outer) has changed
         *        11 - Both switches have changed
         *
         * If there is a change then we need to deduce if we need to change switch state or not.
         */

		j = KeyLoc[i].Sw1.Col; /* column number of switch 1   */
		k = KeyLoc[i].Sw1.Row; /* row number of switch 1 */
		l = KeyLoc[i].Sw2.Col; /* column number of switch 2    */
		m = KeyLoc[i].Sw2.Row; /* row number of switch 2  */

        /* bit 0 of change is switchstatus of Switch 1, bit 1 is switchstatus of switch 2 */
		change = (  ((SwitchStatus[k]._ColumnData >> (j)) & 0x1) |
                    (( (SwitchStatus[m]._ColumnData >> (l)) << 1 ) & 0x2) );  


		switch(KeyStatus[i].State)
		{
			case ( KEY_STATE00 ):
				switch(change)
				{
					case (0x01):
						KeyStatus[i].State = KEY_STATE01;
						KeyStatus[i].Count = 1;
					break;

					case (0x02):
						KeyStatus[i].State = KEY_STATE02;
                        KeyStatus[i].MidiCode = (unsigned char)(MIDIKeyCode[i] + Transposition);
                        //KeyOnCount++;
                        //MIDIAllOffSent = FALSE;
						MIDIOn( &KeyStatus[i]);
					break;

					case (0x03):
						KeyStatus[i].State = KEY_STATE03;
                        KeyStatus[i].MidiCode = (unsigned char)(MIDIKeyCode[i] + Transposition);
                        //KeyOnCount++;
                        //MIDIAllOffSent = FALSE;
						MIDIOn( &KeyStatus[i]);
					break;
				}
				break;

			case ( KEY_STATE01 ):
				switch(change)
				{
					case (0x01):
						KeyStatus[i].State = KEY_STATE00;
						KeyStatus[i].Count = 0;
					break;
					case (0x02):
						KeyStatus[i].State = KEY_STATE03;
						KeyStatus[i].MidiCode = (unsigned char)(MIDIKeyCode[i] + Transposition);
                        //KeyOnCount++;
                        //MIDIAllOffSent = FALSE;
						MIDIOn( &KeyStatus[i]);
						KeyStatus[i].Count = 0;
					break;
					case (0x03):
						KeyStatus[i].State = KEY_STATE02;
						KeyStatus[i].MidiCode = (unsigned char)(MIDIKeyCode[i] + Transposition);
                        //KeyOnCount++;
                        //MIDIAllOffSent = FALSE;
						MIDIOn( &KeyStatus[i]);
						KeyStatus[i].Count = 0;
					break;
				}
			break;

			case ( KEY_STATE02 ):
				switch(change)
				{
					case (0x01):
						KeyStatus[i].State = KEY_STATE03;
					break;
					case (0x02):
						KeyStatus[i].State = KEY_STATE00;
                        //KeyOnCount--;
						MIDIOff( &KeyStatus[i]);
					break;
					case (0x03):
						KeyStatus[i].State = KEY_STATE04;
					break;
				}
			break;

			case ( KEY_STATE03 ):
				switch(change)
				{
					case (0x01):
						KeyStatus[i].State = KEY_STATE05;
					break;
					case (0x02):
						KeyStatus[i].State = KEY_STATE04;
					break;

					case (0x03):
						KeyStatus[i].State = KEY_STATE00;
                        //KeyOnCount--;
						MIDIOff( &KeyStatus[i]);
					break;
				}
			break;

			case ( KEY_STATE04 ):
				switch(change)
				{
					case (0x01):
						KeyStatus[i].State = KEY_STATE00;
                        //KeyOnCount--;
						MIDIOff( &KeyStatus[i]);
					break;

					case (0x02):
						KeyStatus[i].State = KEY_STATE03;
					break;
					case (0x03):
						KeyStatus[i].State = KEY_STATE02;
					break;
				}
			break;

			case ( KEY_STATE05 ):
				switch(change)
				{
					case (0x01):
						KeyStatus[i].State = KEY_STATE03;
					break;

					case (0x02):
						KeyStatus[i].State = KEY_STATE00;
                        //KeyOnCount--;
						MIDIOff( &KeyStatus[i]);
					break;

					case (0x03):
						KeyStatus[i].State = KEY_STATE04;
					break;
				}
			break;

			default:
				KeyStatus[i].State = KEY_STATE00;
			break;
		}   /* 	switch(KeyStatus[i].State)*/

	    /* Increment all non-zero counters */
	    if((KeyStatus[i].Count < MAX_SCAN_COUNT) && (KeyStatus[i].Count))
	    {
		    KeyStatus[i].Count++;
	    }

	}   /* 	for (i = 0; i < NUM_KEYS; i++)*/

    /* Transposition switches */

	/* Check for transposition timeout  */
    if (!--TCount)
    {          
        /* read key */
        if ( Key == SWITCH_T_UP)
        {
	        switchNew[Key] = mUPTSwitchState();
        }
        else
        {
	        switchNew[Key] = mDOWNTSwitchState();
        }

        /*switch on switch state */
       	switch(switchState[Key])
        {
            default:
       	    case SWITCH_WAIT:       /* Waiting for new status change (indeterminate state) */
       	    case SWITCH_WAIT_UP:    /* Waiting for new transistion from UP to DOWN */
       	    case SWITCH_WAIT_DOWN:  /* Waiting for new transistion from DOWN to UP */
           	    /*if switch status changes*/
               	if (switchNew[Key] != switchLast[Key])
               	{
                    /*save key pressed */
       	            switchLast[Key] = switchNew[Key];

           	        /* debounce this key */
               	    switchDebounceTimeout[Key] = DEBOUNCE_COUNT;
                   	switchState[Key] = SWITCH_DEBOUNCE;
    
                }/*if switch status changes*/
                
            break;
                
           	case SWITCH_DEBOUNCE:/* Debouncing key */
            	/* If key still pressed */
               	if (switchNew[Key] == switchLast[Key])
                {
                    /*if switch debounced*/
       	            if (!--switchDebounceTimeout[Key])
       	            {

      					/* Save switch state */
       	                switchNew[Key] = switchLast[Key];

                       /* set key pressed */
    					if (switchNew[Key] == SWITCH_PRESSED)
    					{

                            /* wait for next switch change */
              	            switchState[Key] = SWITCH_WAIT_DOWN;
                            
                            if ( KeyboardMode == KEYBOARD_MODE_MIDI)
                            {
                                /* Check here for both keys pressed (Key == SWITCH_T_UP) and ( Key == SWITCH_T_DOWN) */
    					        if (switchState[(Key+1)%NUM_T_SWITCHES] == SWITCH_WAIT_DOWN)
                                {   /* Both keys are pressed symultaneously so reset transposition*/
                                    OctaveTransposition = 0;
                                    MIDIAllOff();
                                }
   					            else if ( Key == SWITCH_T_UP)
                                {   /* Transposition up */
                                    if (++OctaveTransposition > MAX_TRANSPOSITON)
                                        OctaveTransposition = MAX_TRANSPOSITON;
                                }
                                else
                                {   /* Transposition down */
                                    if (--OctaveTransposition < -(MAX_TRANSPOSITON))
                                        OctaveTransposition = -(MAX_TRANSPOSITON);
                                }

                                Transposition = OctaveTransposition * SEMITONES_PER_OCTAVE;
                            }
                            else if ( KeyboardMode == KEYBOARD_MODE_ALT) /* Keyboard in alt mode */
							{
    					        if (switchState[(Key+1)%NUM_T_SWITCHES] == SWITCH_WAIT_DOWN)
                                {   /* Both keys are pressed symultaneously so send All Notes Off */
                                    MIDIAllOff();
                                }
							}
                            else /* Keyboard in test mode */
   					        {
                                if ( Key == SWITCH_T_UP)
                                {   /* Transposition up */
                                    MIDITransUpOn();
                                }
                                else
                                {   /* Transposition down */
                                    MIDITransDownOn();
                                }
                            }            
    					}
                        else
                        {   /* set key release */
                            switchState[Key] = SWITCH_WAIT_UP;
                            /* Not action other than wait for next key press */
                        }
               	    }/*if switch debounced*/
              	}/* if switch still same status */
                else /*else switch status has changed mid debounce so state indeterminate */
   	            {
       	            switchState[Key] = SWITCH_WAIT;
                }

           break;
                   
        }    /*switch on switch state */
        /* End of Switch code */        

        /* Roll around all transposition switches */
        if (++Key >= NUM_T_SWITCHES)
            Key = SWITCH_T_UP;

        /* reload count */
        TCount = TCOUNT_RELOAD; 
    }/* Check for transposition timeout  */
    
    /* Check for keyoff timeout  */
//    if ( KeyOnCount == 0 )
//    {
//        if ((--KeyOffTimeout == 0 )& (!MIDIAllOffSent))
//        {
//            MIDIAllOff();
//            MIDIAllOffSent = TRUE;
//        }
//    }
//    else
 //   {        
 //       KeyOffTimeout = KEYOFF_TIMEOUT;
//    }

}

/* EOF Keygrd.c */
