/*********************************************************************
 * Microchip Technology, Inc. Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p33fxxxx.h>
#include <stdio.h>
#include <ctype.h>
#include <uart.h>

#include "typedefs.h"
#include "main.h"
#include "adcDrv1.h"
#include "io_cfg.h"                                 // Required
#include "timers.h"
#include "keybrd.h"
#include "serial.h"
#include "midi.h"
#include "dee.h"
#include "version.h"

/*****************************************************************************
*   Configuration Variables
******************************************************************************/
_FBS(RBS_NO_RAM & BSS_NO_FLASH & BWRP_WRPROTECT_OFF)
_FSS(RSS_NO_RAM & SSS_NO_FLASH & SWRP_WRPROTECT_OFF)
_FGS(GSS_OFF    & GCP_OFF      & GWRP_OFF)
_FOSCSEL(FNOSC_FRCPLL & IESO_ON)
_FOSC(FCKSM_CSDCMD & OSCIOFNC_ON & POSCMD_NONE)
_FWDT(FWDTEN_OFF & WINDIS_OFF & WDTPRE_PR128 & WDTPOST_PS32768)
_FPOR(FPWRT_PWR128)


#pragma code
/** P R I V A T E  P R O T O T Y P E S ***************************************/
//extern dword ReadLatch(word, word);
//extern dword WriteLatch(word, word, word, word);
extern void ResetDevice(void);
extern void WriteMem(word);
extern void Erase(word, word ,word);
void InitializeSystem(void);
void EraseBootloadVector(void);
void GoResetDevice(void);
void SendResponse(tResponseType ResponseType );
unsigned char WRITESerialNo (void);
unsigned char READSerialNo (void);

/** V E C T O R  R E M A P P I N G *******************************************/


/** D E C L A R A T I O N S **************************************************/
#define BOOTLOAD_WORD_ADDR  0x1000    /* address of byte to be checked by bootload software  */
#define BOOTLOAD_BYTE       0xFF

#define PROGRAM_WORD_WRITE  0x4003
#define PM_ROW_ERASE 		0x4042
#define PM_ROW_WRITE 		0x4001

#define STATUS_BYTE         0xF0        /* from B0  */
#define EOX_BYTE            0xF7        /* from 0x00 */

#define SPACE               0x20
#define DASH                0x2D

/* MAnufacturers MIDI id bytes */
#define ID1                 0x00
#define ID2                 0x20 
#define ID3                 0x6D

/* response codes  */
#define CODE_ACK            0x7F
#define CODE_NACK           0x7E
#define CODE_VER_NO         0x7D
#define CODE_SER_NO         0x7C
#define CODE_MODE           0x7B

#define NUM_STATUS_BYTES    0x02
#define NUM_SERIAL_BYTES    0x20        /* 32 */

#define EE_STATUS_ADDRESS   0x0000
#define EE_SERIAL_ADDRESS   (EE_STATUS_ADDRESS + NUM_STATUS_BYTES)
#define EE_MODE_ADDRESS     (EE_SERIAL_ADDRESS + NUM_SERIAL_BYTES)

typedef enum
{
    WAIT_COMMAND,
    WAIT_ID1,
    WAIT_ID2,
    WAIT_ID3,
    WAIT_SYSCOM,
    WAIT_SER_NO_DATA,
    WAIT_EOX
}   tCommandStatus;

typedef enum
{
    SYSCOM_NONE,
    SYSCOM_TESTMODE,
    SYSCOM_APP_ENABLE,
    SYSCOM_APP_DISABLE,
    SYSCOM_BOOTLOAD,          // Changed from 0x78,
    SYSCOM_SERIAL_WRITE,
    SYSCOM_VERSION_READ,
    SYSCOM_SERIAL_READ,
    SYSCOM_MODE_READ,
    SYSCOM_MODE_ERASE,		// Used in programming mode only
    SYSCOM_MODE_PROG,		// Used in programming mode only
    SYSCOM_MIDIMODE,		// Set standard MIDI mapping mode
    SYSCOM_ALTMODE			// Set alernative MIDI mapping mode
}   tCommandPending;

typedef enum
{
    SYSSTATUS_BOOTMODE,
    SYSSTATUS_TESTMODE,
    SYSSTATUS_APP_ENABLED,
    SYSSTATUS_APP_DISABLED,
    SYSSTATUS_ILLEGAL = 0x00FF,
}   tSystemStatus;

typedef enum
{
	SYSMODE_MIDI,
	SYSMODE_ALT,
}   tSystemMode;

#define SYSSSTATUS_FLAGS 0x00FF
#define SYSMODE_FLAGS    0x00FF

tSystemStatus SystemStatus;		/* System configuration status */
tSystemMode   SystemMode;		/* System mode */

tTSwitch KeyUPT;
tTSwitch KeyDOWNT;

byte iSerialNumber;
char SerialNumber[NUM_SERIAL_BYTES];

/** D E C L A R A T I O N S **************************************************/
#pragma code
/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
int main ( void )
{
    byte InData;
	BOOLEAN NULLFlag = FALSE;

    tCommandStatus CommandStatus = WAIT_COMMAND;
    tCommandPending CommandPending = SYSCOM_NONE;

    DWORD SourceAddress;

    SourceAddress._dword = BOOTLOAD_WORD_ADDR;

    SET_CPU_IPL(7);

    DataEEInit();
    InitializeSystem();

    SET_CPU_IPL(0);

    /* Check AppStatus   */
    SystemStatus = (tSystemStatus)( DataEERead(EE_STATUS_ADDRESS) & SYSSSTATUS_FLAGS);
    SystemMode   = (tSystemMode)  ( DataEERead(EE_MODE_ADDRESS)   & SYSMODE_FLAGS);

    /* Force system configuration depending on stored SytemStatus */
    if (SystemStatus == SYSSTATUS_ILLEGAL)
    {   /* Force SYSSTATUS_APP_DISABLED */ 
        SystemStatus = SYSSTATUS_APP_DISABLED; 
        DataEEWrite( (unsigned int) SYSSTATUS_APP_DISABLED, EE_STATUS_ADDRESS ); /*  Write disable byte to flash memory  */
    }
    else if (SystemStatus == SYSSTATUS_APP_ENABLED)
    {
		// Check for keyboard mode over-ride here

        KeyUPT   = mUPTSwitchState();
        KeyDOWNT = mDOWNTSwitchState();

		if ((KeyUPT == SWITCH_PRESSED) && (KeyDOWNT != SWITCH_PRESSED))
		{
			if ( SystemMode != SYSMODE_MIDI)
			{
		        DataEEWrite( (unsigned int) SYSMODE_MIDI, EE_MODE_ADDRESS ); /*  Write mode byte to flash memory  */
				SystemMode = SYSMODE_MIDI;
			}
			while (KeyUPT == SWITCH_PRESSED)
			{
		        KeyUPT = mUPTSwitchState();
			}
		}
		else if ((KeyUPT != SWITCH_PRESSED) && (KeyDOWNT == SWITCH_PRESSED))
		{
			if ( SystemMode != SYSMODE_ALT)
			{
		        DataEEWrite( (unsigned int) SYSMODE_ALT, EE_MODE_ADDRESS ); /*  Write mode byte to flash memory  */
				SystemMode = SYSMODE_ALT;
			}
			while (KeyDOWNT == SWITCH_PRESSED)
			{
		        KeyDOWNT = mDOWNTSwitchState();
			}
		}

		// Set for MIDI/ALT keyboard mode
		if (SystemMode == SYSMODE_ALT)
		{
	        KEYBRDInit(KEYBOARD_MODE_ALT);
		}
		else
		{
	        KEYBRDInit(KEYBOARD_MODE_MIDI);	// Always default to MIDI mode
			if (SystemMode != SYSMODE_MIDI)
			{	/* Must be able to cope with unitialised EE ROM */
		        DataEEWrite( (unsigned int) SYSMODE_MIDI, EE_MODE_ADDRESS ); /*  Write mode byte to flash memory  */
				SystemMode = SYSMODE_MIDI;
			}
		}
   	    initAdc1();             	// Initialize the A/D converter to convert Channel 5
	    initDma0();					// Initialise the DMA controller to buffer ADC data in conversion order
        TIMER1Init();               // System tick
    }
   
    SERIALInit();

    while(1)
    {        
        if (!SERIALInBufferIsEmpty())
        {
            InData = SERIALReadByte();
  
            switch(CommandStatus)
            {
                case WAIT_COMMAND:   /* waiting for a command string to start i.e status byte  */
                    if (InData == STATUS_BYTE)  /* valid command start byte */
                        CommandStatus = WAIT_ID1;    /* wait for valid id */
                break;

                case WAIT_ID1:   /* waiting for first Id byte  */
                    if (InData == ID1)  /* valid ID1 byte */
                        CommandStatus = WAIT_ID2;    /* wait for valid id */
                    else
                        CommandStatus = WAIT_COMMAND;    /* back to beginning */
                break;

                case WAIT_ID2:   /* waiting for second Id  byte  */
                    if (InData == ID2)  /* valid second Id  byte */
                        CommandStatus = WAIT_ID3;    /* wait for valid id */
                    else
                        CommandStatus = WAIT_COMMAND;    /* back to beginning */
                break;

                case WAIT_ID3:   /* waiting for Id string to start i.e status byte  */
                    if (InData == ID3)  /* valid third Id byte */
                        CommandStatus = WAIT_SYSCOM;    /* wait for valid command */
                    else
                        CommandStatus = WAIT_COMMAND;    /* back to beginning */
                break;

                case WAIT_SYSCOM:   /* Waiting for a valid command byte */
                {
                    switch(InData)
                    {
                        case SYSCOM_TESTMODE:   /*  testmode command receieved  */
                            CommandPending = SYSCOM_TESTMODE;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_APP_ENABLE:   /*  enable application command receieved  */
                            CommandPending = SYSCOM_APP_ENABLE;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_MIDIMODE:      /*  Select stanard MIDI mapping mode */
                            CommandPending = SYSCOM_MIDIMODE;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_ALTMODE:      /*  Select alternative MIDI mapping mode */
                            CommandPending = SYSCOM_ALTMODE;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_APP_DISABLE:   /* disable application command receieved  */
                            CommandPending = SYSCOM_APP_DISABLE;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_BOOTLOAD:   /*  Bootload command receieved  */
                            CommandPending = SYSCOM_BOOTLOAD;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_SERIAL_WRITE:   /*  write serial command receieved  */
                            CommandPending = SYSCOM_SERIAL_WRITE;
                            CommandStatus = WAIT_SER_NO_DATA;
                            /* initialize Serial number array to recieve data */
                            for (iSerialNumber  = 0; iSerialNumber < NUM_SERIAL_BYTES; iSerialNumber++)
                            {
                                SerialNumber[iSerialNumber] = 0;
                            }
                            iSerialNumber = 0;  /* initialise index */
							NULLFlag = FALSE;
                        break;

                        case SYSCOM_VERSION_READ:   /* Read version command receieved  */
                            CommandPending = SYSCOM_VERSION_READ;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_SERIAL_READ:
                            CommandPending = SYSCOM_SERIAL_READ;
                            CommandStatus = WAIT_EOX;
                        break;

                        case SYSCOM_MODE_READ: 
                            CommandPending = SYSCOM_MODE_READ;
                            CommandStatus = WAIT_EOX;
                        break;

                        default:    /* valid command not received, start wait for command again */
                            CommandPending = SYSCOM_NONE;
                            CommandStatus = WAIT_COMMAND;
                        break;

                    } /*    switch(InData)  */

                }   
                break;  /* case WAIT_SYSCOM */

                case WAIT_SER_NO_DATA:   /* waiting for 32 bytes of serial number  */
                    if (iSerialNumber < NUM_SERIAL_BYTES)
                    {
						if (InData == NULL)
						{
							NULLFlag = TRUE;
                            SerialNumber[iSerialNumber++] = InData;           
                            if  (iSerialNumber == NUM_SERIAL_BYTES)   /* if 32 bytes data received now looking for EOX  */
                                CommandStatus = WAIT_EOX;       
						}
                        else if (((isalnum((int)InData))
                           ||  (InData == SPACE)
                           ||  (InData == DASH))
						   &&  (NULLFlag == FALSE))
                        { /* if  valid serial digits  */
                            SerialNumber[iSerialNumber++] = InData;           
                            if  (iSerialNumber == NUM_SERIAL_BYTES)   /* if 32 bytes data received now looking for EOX  */
                                CommandStatus = WAIT_EOX;       
                        }
                        else /*not valid serial digits end serial number write*/
                        {
							SendResponse( RESPONSE_NACK ); /* send NACK  */  							
                            CommandPending = SYSCOM_NONE;
                            CommandStatus = WAIT_COMMAND;   
                        }
                    }
                break;

                case WAIT_EOX:   /* Waiting for final byte */
                {
                    if ( InData == EOX_BYTE )
                    {
                        switch(CommandPending)
                        {
                            case SYSCOM_TESTMODE:   /*  SYSCOM_TESTMODE command receieved  */
                                SystemStatus = SYSSTATUS_TESTMODE; 
								NextKeyboardScan = NULL;
								TIMER1Reset(); 
								resetAdc1();
								resetDma0();
                                KEYBRDInit(KEYBOARD_MODE_TEST);   // Test mode is always volatile so don't store the mode
                                initAdc1();             	// Initialize the A/D converter to convert Channel 5
	                            initDma0();					// Initialise the DMA controller to buffer ADC data in conversion order
                                TIMER1Init();
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                                SendResponse( RESPONSE_ACK ); /* send ACK  */
                            break;
   
                            case SYSCOM_APP_ENABLE:   /*  SYSCOM_APP_ENABLE command receieved  */                             
                                SystemStatus = SYSSTATUS_APP_ENABLED;                                 
                                DataEEWrite( (unsigned int) SYSSTATUS_APP_ENABLED, EE_STATUS_ADDRESS ); /*  Write enable byte to flash memory  */
                                NextKeyboardScan = NULL;
								TIMER1Reset(); 
								resetAdc1();
								resetDma0();
								KEYBRDInit(KEYBOARD_MODE_MIDI); // Always put application back into MIDI decode mode
                                initAdc1();             	// Initialize the A/D converter to convert Channel 5
	                            initDma0();					// Initialise the DMA controller to buffer ADC data in conversion order
                                TIMER1Init();   
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;  
                                SendResponse( RESPONSE_ACK ); /* send ACK  */  
                            break;

                            case SYSCOM_APP_DISABLE:   /* SYSCOM_APP_DISABLE command receieved  */
                                SystemStatus = SYSSTATUS_APP_DISABLED; 
                                DataEEWrite( (unsigned int) SYSSTATUS_APP_DISABLED, EE_STATUS_ADDRESS ); /*  Write disable byte to flash memory  */                                
          	                    TIMER1Reset();  /* Stop ADC dma and timer  */
								NextKeyboardScan = NULL;
                                SendResponse( RESPONSE_ACK ); /* send ACK  */
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                            break;

	                        case SYSCOM_MIDIMODE:      /*  Select alternative MIDI mapping mode */
                                NextKeyboardScan = NULL;
								TIMER1Reset(); 
								resetAdc1();
								resetDma0();
								KEYBRDInit(KEYBOARD_MODE_MIDI); // Force MIDI decode mode
						        DataEEWrite( (unsigned int) SYSMODE_MIDI, EE_MODE_ADDRESS ); /*  Write mode byte to flash memory  */
                                initAdc1();             	// Initialize the A/D converter to convert Channel 5
	                            initDma0();					// Initialise the DMA controller to buffer ADC data in conversion order
                                TIMER1Init();   
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;  
                                SendResponse( RESPONSE_ACK ); /* send ACK  */  
							break;

    	                    case SYSCOM_ALTMODE:      /*  Select alternative MIDI mapping mode */
                                NextKeyboardScan = NULL;
								TIMER1Reset(); 
								resetAdc1();
								resetDma0();
								KEYBRDInit(KEYBOARD_MODE_ALT); // Force Alternative decode mode
						        DataEEWrite( (unsigned int) SYSMODE_ALT, EE_MODE_ADDRESS ); /*  Write mode byte to flash memory  */
                                initAdc1();             	// Initialize the A/D converter to convert Channel 5
	                            initDma0();					// Initialise the DMA controller to buffer ADC data in conversion order
                                TIMER1Init();   
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;  
                                SendResponse( RESPONSE_ACK ); /* send ACK  */  
							break; 

                            case SYSCOM_BOOTLOAD:   /*  SYSCOM_BOOTLOAD command receieved  */
                               /* This erases the program memory at BOOTLOAD_WORD_ADDR */  
                                EraseBootloadVector();
							
								CloseUART2();

                                /* Go to bootstrap code */
                                GoResetDevice();
                            break;

                            case SYSCOM_SERIAL_WRITE:   /*   SYSCOM_SERIAL_WRITE command receieved  */   

                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;   
                                if (WRITESerialNo ())   /* error returned */
                                {
                                    SendResponse( RESPONSE_NACK ); /* send NACK  */  
                                }
                                else    /* no error returned */
                                {
                                    SendResponse( RESPONSE_ACK ); /* Output serial number  */  
                                }
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                            break;

                            case SYSCOM_VERSION_READ:   /*  SYSCOM_VERSION_READ command receieved  */    
                                SendResponse( RESPONSE_VER_NO );
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                            break;

                            case SYSCOM_SERIAL_READ:
                                if (READSerialNo ())
                                {
                                     SendResponse( RESPONSE_SER_NO ); /* send SER_NO response  */  
                                }
                                else    /* error returned */
                                {
                                    SendResponse( RESPONSE_NACK ); /* send NACK  */  
                                }
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                            break;

                            case SYSCOM_MODE_READ:  
                                SendResponse( RESPONSE_MODE );
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                            break;

                            default:    /* invalid command pending, start wait for command again */
                                CommandPending = SYSCOM_NONE;
                                CommandStatus = WAIT_COMMAND;
                            break;

                        } /*    switch(CommandPending)  */
                    }   /* if ( InData == EOX_BYTE ) */
                    else    /* didn't receive EOX_BYTE */
                    {
                        CommandPending = SYSCOM_NONE;
                        CommandStatus = WAIT_COMMAND;
                    }


                }   /* WAIT_EOX */

                default:    /* ignore any other input */

                break;
            }   /* switch(CommandStatus) */
        }  /*  if (!SERIALInBufferEmpty()) */


       	if (NextKeyboardScan != NULL)
		{
            mLEDRed_Toggle();   /* debug  */
			KEYBRDStateMachine();
			NextKeyboardScan = NULL;
		}
    } //end while

}//end main

/******************************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization routine.
 *                  All required USB initialization routines are called from
 *                  here.
 *
 *                  User application initialization routine should also be
 *                  called from here.                  
 *
 * Note:            None
 *****************************************************************************/
void InitializeSystem(void)
{
 	/* Configure CLKDIV for Maximum Fcy */

    /*
    OSCILLATOR RULES:
    A) Output from the PLLPRE "MUST" be in the range of (0.8-8.0 MHz) at all times
    B) Output from the VCO, (PLLDIV), "MUST" be in the range of (100-200 MHz) at all times
    C) Output from the PLLPOST "MUST" be in the range of (12.5-80 MHz) at all times
    D) Violating any rule while the PLL is active and the oscillator will become unstable 
       and is not guaranteed to return to normal even if values are later corrected in 
       CLKDIV & PLLFBD to be within range.

    Using internal fast RC @ 7.37MHz
    At power on:
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 184.25Mhz = OK
      //NOTE:Output from the PLLPOST = 46.0625Mhz = OK


    Default POR values:
    PLLPRE = 0; ...........(0+2) = 2
    PLLDIV = 0x30 = 48;....(48+2)= 50
    PLLPOST = 1; ..........(4)

    Final configuration 
    PLLPRE = 0; ...........(0+2) = 2
    PLLDIV = 0x26 = 38;....(38+2)= 40
    PLLPOST = 0; ..........(2)
    */

    CLKDIVbits.PLLPRE  = 0;	/* Pre Divide by 2 */
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 184.25Mhz = OK
      //NOTE:Output from the PLLPOST = 46.0625Mhz = OK

    PLLFBDbits.PLLDIV  = 41;	/* PLL Multiply by 43  */
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 147.4Mhz = OK
      //NOTE:Output from the PLLPOST = 46.0625Mhz = OK

    CLKDIVbits.PLLPOST = 0;	/* Post Divide by 2 	*/
      //NOTE:Output from the PLLPRE = 3.685Mhz = OK
      //NOTE:Output from the VCO = 147.4Mhz = OK
      //NOTE:Output from the PLLPOST = 79.2275Mhz = OK

    CLKDIVbits.FRCDIV  = 0;	/* FRC divide by 1 */
      //FRCDIV bits are don't care if PLL mode enabled. 
      //FRCDIV bits are only important if your configuration words are something like
      //  _FOSC( FCKSM_CSDCMD & OSCIOFNC_ON & POSCMD_NONE); 
      //  _FOSCSEL( FNOSC_FRC & IESO_OFF & TEMP_OFF); 

    /* need to tune internal oscillator up by 1.125% */
    OSCTUNbits.TUN = 3;   /*  !!!!  */

    //Wait until PLL has re-locked
    while(OSCCONbits.LOCK != 1);    
    
    mInitTSwitches();               /* initialise transposition switches    */

    LATD = 0;

 	mInitLEDS();					    // LED on


}//end InitializeSystem


/*
** Enable interrupts to CPU
**
*/
static int old_ipl = 0;

void InterruptEnable (void)
{
    RESTORE_CPU_IPL(old_ipl);
}

/*
** Disable interrupts to CPU
**
*/
void InterruptDisable (void)
{
    SET_AND_SAVE_CPU_IPL(old_ipl, 7);
}

void GoResetDevice(void)
{
	ResetDevice();
}


/*****************************************************************************
*
* Name:    SendResponse
*
* Description: Sends response to SYSEX message
*          
*
*****************************************************************************/
void SendResponse(tResponseType ResponseType )
{
    char * VersionString = FW_NUMBER;

    /* send Status byte and manufacturers ID  */
    SERIALWriteByte (STATUS_BYTE);                    
    SERIALWriteByte (ID1);             
    SERIALWriteByte (ID2);            
    SERIALWriteByte (ID3); 

    switch(ResponseType)
    {
        case RESPONSE_ACK:   
            SERIALWriteByte (CODE_ACK);      
        break;

        case RESPONSE_NACK:   
            SERIALWriteByte (CODE_NACK);
        break;

        case RESPONSE_VER_NO:   

            SERIALWriteByte (CODE_VER_NO);      
            SERIALWriteByte (*VersionString++);
            SERIALWriteByte (*VersionString++); 
            SERIALWriteByte (*VersionString++); 
            SERIALWriteByte (*VersionString);           
                 
        break;

        case RESPONSE_SER_NO:   
            SERIALWriteByte (CODE_SER_NO);      
            for (iSerialNumber  = 0; iSerialNumber < NUM_SERIAL_BYTES; iSerialNumber++)
            {
                SERIALWriteByte (SerialNumber[iSerialNumber]);
            }
        break;

        case RESPONSE_MODE:
            SERIALWriteByte (CODE_MODE);      
            switch(SystemStatus)
            {
                case SYSSTATUS_TESTMODE:   
                    SERIALWriteByte ((byte)SYSSTATUS_TESTMODE);
                break;

                case SYSSTATUS_APP_ENABLED:   
                    SERIALWriteByte ((byte)SYSSTATUS_APP_ENABLED);                
                break; 

                case SYSSTATUS_APP_DISABLED:
                    SERIALWriteByte ((byte)SYSSTATUS_APP_DISABLED);
                break; 

                default:   
                    SERIALWriteByte ((byte)SYSSTATUS_ILLEGAL);
                break;
            }
                  
        break;
    }

    /* End of message  */ 
    SERIALWriteByte (EOX_BYTE);         
}


/*****************************************************************************
*
* Name:    WRITESerialNo
*
* Description: Writes ASCII coded Serial number to Flash memory as words
*          
*
*****************************************************************************/
unsigned char WRITESerialNo (void)
{
    unsigned char ReturnCode = 0;
   
    /* Initialize Serial number array to recieve data */
    for (iSerialNumber  = 0; iSerialNumber < NUM_SERIAL_BYTES; iSerialNumber++)
    {
        ReturnCode = DataEEWrite((unsigned int)SerialNumber[iSerialNumber], (EE_SERIAL_ADDRESS+iSerialNumber));
    }
    
    return ReturnCode;
}


/*****************************************************************************
*
* Name:    READSerialNo
*
* Description: Reads serial number data from Flash memory and place in Serial 
* Number array for sending to serial comms  
*          
*
*****************************************************************************/
unsigned char READSerialNo (void)
{   
    /* Read EE into Serial number array */
    for (iSerialNumber  = 0; iSerialNumber < NUM_SERIAL_BYTES; iSerialNumber++)
    {
        SerialNumber[iSerialNumber] = (char) DataEERead(EE_SERIAL_ADDRESS+iSerialNumber);
    }

    if (  (isalnum(SerialNumber[0]))
       || (SerialNumber[0] == SPACE )
       || (SerialNumber[0] == DASH ))
        return (1);
    else
        return (0); /* No serial number set */
}




/**
**  Erase BOOTLOAD_WORD vector in program memory - to be used with extreme caution !!
*/

#define PM_WORD_WRITE 0x4003
extern  void WriteMem(word);
extern void WriteLatch(word, word, word, word);

void EraseBootloadVector(void)
{
    DWORD TempAddr;

    TempAddr._dword = BOOTLOAD_WORD_ADDR;   /* Location of redirection flag */

    InterruptDisable ();

	Erase(TempAddr.word1, TempAddr.word0,PM_ROW_ERASE);

    InterruptEnable ();
}


/** EOF main.c ***************************************************************/

