/*****************************************************************************
* Filename          : midi.c
*
* Module            : 
*
* Author            : Val Tarrant/John Thorne
* Description       : This module holds the MIDI functions for the 
*                     HT-MINI MIDI Keyboard scanning
*	
*
* Change history    :
*   Issue   Date        Changed by      Reason for change
*   1.0     30Jan2008   Val Tarrant     First Issue
*
*
* Copyright         : Cambridge Micro Engineering, Jan 2008
*
******************************************************************************/

/*****************************************************************************
* Library Includes
*****************************************************************************/

#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#include <libpic30.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#include <libpic24.h>
#endif

#include <stdio.h>
#include <string.h>

/*****************************************************************************
* Project Includes
*****************************************************************************/
#include    "typedefs.h"
#include	"midi.h"
#include    "keybrd.h"
#include    "serial.h"
/******************************************************************************
*   Local Constants
******************************************************************************/

/* table to convert count to midi velocity */
byte MIDIVelocity[] =  {
    0x7F, 0x7F, 0x7F, 0x7F, 0x7D, 0x7A, 0x76, 0x72,
    0x6E, 0x6A, 0x66, 0x62, 0x5E, 0x5A, 0x56, 0x52,
    0x4E, 0x4A, 0x46, 0x42, 0x3E, 0x3C, 0x38, 0x34,
    0x30, 0x2C, 0x28, 0x24, 0x20, 0x1F, 0x1E, 0x1D,
    0x1C, 0x1B, 0x1A, 0x19, 0x18, 0x17, 0x16, 0x15,
    0x14, 0x13, 0x12, 0x11, 0x10, 0x0F, 0x0E, 0x0D,
    0x0C, 0x0B, 0x0A, 0x09, 0x08, 0x07, 0x06, 0x05,
    0x04, 0x03, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00
};

#define MIDI_OFF    0x80    /* command 8 + channel 0 */
#define MIDI_ON     0x90    /* command 9 + channel 0 */
#define CHANNEL_MODE    0xB0	/** Midi code for Channel Mode */
#define ALL_NOTES_OFF   0x7B    /** Midi code for All notes Off */
#define MIDI_TUP        0x62	/** Midi code for key up */
#define MIDI_TDOWN      0x63	/** Midi code for key down */


/*****************************************************************************
*   Local Variables
******************************************************************************/

/*****************************************************************************
*   Global Variables
******************************************************************************/

/*****************************************************************************
*   Local Function Prototypes
*****************************************************************************/


/*****************************************************************************
*
* Name:    MIDIInit
*          
* Description:  Initialise MIDI I/O to defaults
*
*
*****************************************************************************/
void MIDIInit (void)
{
}

/*****************************************************************************
*
* Name:    MIDIOn( pointer to status structure )
*
* Description: Keyboard State Machine control
*          
*
*****************************************************************************/
void MIDIOn(tKeyStatus * Key )
{
    SERIALWriteByte (MIDI_ON);                  /* midi on command */
    SERIALWriteByte (Key->MidiCode);             /* midi code for key */
    SERIALWriteByte (MIDIVelocity[Key->Count]);  /* Velocity for key */
}

/*****************************************************************************
*
* Name:    MIDIOff( key )
*
* Description: Keyboard State Machine control
*          
*
*****************************************************************************/
void MIDIOff(tKeyStatus * Key )
{
    SERIALWriteByte (MIDI_OFF);             /* midi off command */
    SERIALWriteByte (Key->MidiCode);         /* midi code for key */
    SERIALWriteByte (0);                    /* off Velocity always zero */
}

/*****************************************************************************
*
* Name:   MIDITransUpOn( void )
*
* Description: Keyboard State Machine control
*          
*
*****************************************************************************/
void MIDITransUpOn(void )
{
    SERIALWriteByte (MIDI_ON);             /* midi off command */
    SERIALWriteByte (MIDI_TUP);         /* midi code for key */
    SERIALWriteByte (0);                    /* off Velocity always zero */
}

/*****************************************************************************
*
* Name:    MIDITransDownOn( key )
*
* Description: Keyboard State Machine control
*          
*
*****************************************************************************/
void MIDITransDownOn(void )
{
    SERIALWriteByte (MIDI_ON);             /* midi off command */
    SERIALWriteByte (MIDI_TDOWN);         /* midi code for key */
    SERIALWriteByte (0);                    /* off Velocity always zero */
}



/*****************************************************************************
*
* Name:    MIDIAllOff
*
* Description: Sends Midi command for all notes off
*          
*
*****************************************************************************/
void MIDIAllOff(void )
{
    SERIALWriteByte (CHANNEL_MODE);                    
    SERIALWriteByte (ALL_NOTES_OFF);             /* midi off command */
    SERIALWriteByte (0);                    
}

/* EOF Midi.c */
