/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include "system\typedefs.h"
#include <usart.h>
#include <string.h>

#include "system\usb\usb.h"
#include "source\io_cfg.h"             // I/O pin mapping
#include "source\user.h"

#include "midi.h"

/** V A R I A B L E S ********************************************************/

/*
** System related variables
*/
#define CLOCK_RATE  (48000000UL)    /**< Nominal clock rate */
#define DSPIC_BAUD  (57600UL)       /**< Baud rate of DS PIC */

/**
**  Baud rate divisor
*/
#define DIVISOR     (((CLOCK_RATE / DSPIC_BAUD) - 1UL) /4UL)

#define DEBUG_LED_ON    LATAbits.LATA5 = 0

#define ProgramMemStart					0x001000 //Beginning of application program memory (not occupied by bootloader).  

/*
** Buffering requirements
*/
#ifndef MIDI_IN_SIZE
#define MIDI_IN_SIZE    (64)        /**< Should be same or less than BULK IN endpoint size */
#endif

#if ((MIDI_IN_SIZE % USBMIDI_LENGTH) != 0)
#error "MIDI_IN_SIZE must be a factor of USBMIDI_LENGTH"
#endif

#define MIDI_CABLE  (0)             /**< USB MIDI cable identifier */
#define MIDI_BOOT_CABLE  (0x0F)		/**< USB MIDI boot cable identifier */

/**
** Possible states of MIDI data received from USB
*/
typedef enum
{
    MIDIRX_STATE_IDLE,      /**< Doing nothing */
    MIDIRX_STATE_HEADER,    /**< Reading CIN */
    MIDIRX_STATE_DATA       /**< Transferring CIN data */
} eMidiRxState;

/**
** Possible states of MIDI data transmitted to USB
*/
typedef enum
{
    MIDITX_STATE_IDLE,      /**< Waiting for a status byte */
    MIDITX_STATE_DATA,      /**< Waiting for data bytes */
    MIDITX_STATE_IDLE_RUN,  /**< Waiting for a status byte or running status */
    MIDITX_STATE_SYSTEM,    /**< System exclusive read */
    MIDITX_STATE_ERROR      /**< Error during input processing */
} eMidiTxState;

/** V A R I A B L E S ********************************************************/

eMidiRxState    rxState;            /**< Current receive (USB OUT) state */
byte            rxIndex;            /**< Current index inot endpoint buffer */
byte            rxSubIndex;         /**< Added index */
byte            rxLimit;            /**< Index limit */

eMidiTxState    txState;                    /**< Current transmit (USB IN) state */
byte            txMidiIn [MIDI_IN_SIZE];    /** Packetised MIDI data: USB MIDI */
byte            txIndex;                    /**< Index into @ref txMidiIn */
byte            txMidiBuffer [USBMIDI_LENGTH];  /** Partial MIDI command */
byte            txMidiIndex;                /**< Index into @ref txMidiBuffer */
byte            txDataLength;               /**< Expected data length */
byte            txRunningStatus;            /**< Last running status */

/**
**  Lengths of MIDI data received from USB
*/
static const byte cinLength [] =
{
    0,      /* 0 */
    0,
    2,
    3,
    3,      /* 4 */
    1,
    2,
    3,
    3,      /* 8 */
    3,
    3,
    3,
    2,      /* C */
    2,
    3,
    1
};

/**
**  Lengths of non-system MIDI commands
*/
static const byte midiLength [] =
{
    2,
    2,
    2,
    2,
    1,
    1,
    2,
    0       /* System command not handled here */
};

/**
**  Lengths of system common commands with one byte added
*/
static const byte midiSystemLength [] =
{
    0,          /* Variable length */
    1 + 1,      /* MTC */
    1 + 2,      /* Song position pointer */
    1 + 1,      /* Song select */
    1 + 0,      /* Undefined */
    1 + 0,      /* Undefined */
    1 + 0,      /* Tune request */
    1 + 0       /* End of exclusive */
};

/**
**  CIN values for system common messages (excluding SysEx start/continue)
*/
static const byte cinMap [] =
{
    0x05,       /* Single byte */
    0x02,       /* Two byte */
    0x03        /* Three byte */
};

/** P R I V A T E  P R O T O T Y P E S ***************************************/

void InitializeUSART(void);
void BlinkUSBStatus(void);
void CheckSerialOverrun (void);
void SendPacketisedData (void);
void PacketiseData (void);
void HandleRealtimeSystem(byte data);
BOOL HandleInputByte(byte data);
void AddMidiBuffer (void);
void DepacketiseData (void);
void EraseBootByte(void);
void InitialiseBootloader (void);

void UnlockAndActivate(void);

/** D E C L A R A T I O N S **************************************************/

/**
**  Initialise user sub-system
*/
void UserInit(void)
{
    mInitLED();

    InitializeUSART();

    /* Release RESET to DS PIC */
    TRISCbits.TRISC2 = 0;
    LATCbits.LATC2   = 1;

    /* Debug LED to indicate overflow */
    LATAbits.LATA5 = 1; TRISAbits.TRISA5 = 0;

    rxState = MIDIRX_STATE_IDLE;
    txState = MIDITX_STATE_IDLE;
    txIndex = 0;
    txRunningStatus = 0;
}

/**
**  Initialise USART ready for use
*/
void InitializeUSART(void)
{

    TRISCbits.TRISC7=1; // RX
    TRISCbits.TRISC6=0; // TX

    SPBRG   =  DIVISOR       & 0xff;
    SPBRGH  = (DIVISOR >> 8) & 0xff;
    TXSTA = 0x24;       // TX enable BRGH=1
    RCSTA = 0x90;       // continuous RX
    BAUDCON = 0x08;     // BRG16 = 1

	/* Enable RTS/CTS Flow Control I/O */
	//TRISBbits.TRISB0 = 1;	/* CTS is input */
	//TRISBbits.TRISB1 = 0;	/* RTS is output */
	//LATBbits.LATB1 = 1;		/* Ready	*/

}

/**
** See serial over-run has occurred
*/
void CheckSerialOverrun (void)
{
    if (RCSTAbits.OERR)   /* re-enable after overrun error */
    {
        RCSTAbits.CREN = 0;
        RCSTAbits.CREN = 1;

        /* Revert to IDLE state */
        txState = MIDITX_STATE_IDLE;
        txRunningStatus = 0;
        DEBUG_LED_ON;
    }
}

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{
    BlinkUSBStatus();

    /* User application commands only run in configured state */
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

    /*
    ** Overrun requires discarding of any partial state.
    */
    CheckSerialOverrun ();

    SendPacketisedData ();

    PacketiseData ();

    DepacketiseData ();

}//end ProcessIO

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    static word led_counter = 10000U;

    if(led_count == 0)led_count = led_counter;
    led_count--;

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_Toggle();
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_On();   /* Should get here is bus powered ! */
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            led_counter = 40000U;   /* Flash very slowly */

            if(led_count == 0)
            {
                mLED_Toggle();
            }//end if
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            led_counter = 20000U;   /* Flash slowly */

            if(led_count == 0)
            {
                mLED_Toggle();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            led_counter = 10000U;   /* Flash normally */

            if(led_count==0)
            {
                mLED_Toggle();
            }//end if
        }//end if(...)
    }//end if(...)
}//end BlinkUSBStatus

/**
**	SendPacketisedData
**
**  Send any accumulated data to USB IN pipe.  The data to be
**  transferred must be sent in USB-MIDI format using MidiTxData
**
*/
void SendPacketisedData (void)
{
    /* Any data to send? */
    if (txIndex > 0)
    {
        if (MidiTxData(txMidiIn, txIndex))
        {
            /* Data taken */
            txIndex = 0;
        }
    }
}

/**
**	PacketiseData
**
**  Packetise data received from UART ready for transmission on USB
**
*/
void PacketiseData (void)
{
    /*
    ** Read and process incoming data
    */
    while (PIR1bits.RCIF)
    {
        /* Read byte to clear receive indication */
        byte    data = RCREG;

        /*
        ** Look for System real-time messages, these don't disturb the
        ** current processing state.
        */
        if (((data & MIDI_MESSAGE_MASK) == MIDI_MESSAGE_SYSTEM)
        &&  (data & MIDI_STATUS_REALTIME))
        {
            HandleRealtimeSystem(data);
        }
        else
        {
            /* Process byte (may repeat to change transmit state machine */
            while (HandleInputByte(data))
            {
                ;
            }
        }
    }
}

/**
**   Add a system realtime message to the USB IN buffer
**
**  @param  data    Byte to process
*/
void HandleRealtimeSystem(byte data)
{
    if (txIndex < MIDI_IN_SIZE)
    {
        txMidiIn [txIndex] = (MIDI_CABLE << USBMIDI_CABLE_SHIFT) | (USBMIDI_CIN_SINGLE << USBMIDI_CIN_SHIFT);
        txMidiIn [txIndex + 1] = data;
        txMidiIn [txIndex + 2] = 0;
        txMidiIn [txIndex + 3] = 0;
        txIndex += USBMIDI_LENGTH;
    }
    else
    {
        /* Buffer overrun: drop on floor */
        DEBUG_LED_ON;
    }
}

/**
**   Add a data message to the USB IN buffer
**
**  @param  data    Byte to process
*/
BOOL HandleInputByte(byte data)
{
    BOOL    retry = FALSE;

    switch (txState)
    {
    case MIDITX_STATE_IDLE:
        /* Only looking for a status byte */
        if (data & MIDI_STATUS_BIT)
        {
            if ((data & MIDI_MESSAGE_MASK) == MIDI_MESSAGE_SYSTEM)
            {
                /* A system message cancels any running status */
                txRunningStatus = 0;

                /* Expected number of bytes for a system message */
                txDataLength = midiSystemLength [data & MIDI_SUBMESSAGE_MASK & ~MIDI_STATUS_REALTIME];

                /* Store command */
                memset (txMidiBuffer, 0, USBMIDI_LENGTH);
                txMidiBuffer [USB_MIDI_DATA_OFFSET] = data;

                /* System Exclusive handled specially */
                if (0 == txDataLength)
                {
                    txMidiIndex = 1;
                    txState = MIDITX_STATE_SYSTEM;
                }
                else
                {
                    /* Form USBMIDI command */
                    txDataLength--;
                    txMidiBuffer [0] = (MIDI_CABLE << USBMIDI_CABLE_SHIFT) | (cinMap[txDataLength] << USBMIDI_CIN_SHIFT);
                    if (0 == txDataLength)
                    {
                        /* Complete at once, remain in IDLE state */
                        AddMidiBuffer ();
                    }
                    else
                    {
                        /* Complete using normal data state */
                        txMidiIndex = 0;
                        txState = MIDITX_STATE_DATA;
                    }
                }
            }
            else
            {
                /* Store new running status and extract expected length */
                txRunningStatus = data;
                txMidiIndex = (data & MIDI_MESSAGE_MASK) >> MIDI_MESSAGE_SHIFT;
                /* Extract length of command */
                txDataLength = midiLength [txMidiIndex & ~(MIDI_STATUS_BIT >> MIDI_MESSAGE_SHIFT)];

                memset (txMidiBuffer, 0, USBMIDI_LENGTH);
                txMidiBuffer [0] = (MIDI_CABLE << USBMIDI_CABLE_SHIFT) | (txMidiIndex << USBMIDI_CIN_SHIFT);
                txMidiBuffer [USB_MIDI_DATA_OFFSET] = data;
                txMidiIndex = 0;
                txState = MIDITX_STATE_DATA;
            }
        }
        /* Drops non-status bytes silently */
        break;

    case MIDITX_STATE_DATA:
        if (data & MIDI_STATUS_BIT)
        {
            txState = MIDITX_STATE_ERROR;
            retry = TRUE;
        }
        else
        {
            txMidiBuffer [USB_MIDI_DATA_OFFSET + 1 + txMidiIndex] = data;
            txMidiIndex++;
            if (txMidiIndex >= txDataLength)
            {
                AddMidiBuffer ();
                txState = MIDITX_STATE_IDLE_RUN;
            }
        }
        break;

    case MIDITX_STATE_IDLE_RUN:
        if (data & MIDI_STATUS_BIT)
        {
            /* New status so re-enter through IDLE state */
            txState = MIDITX_STATE_IDLE;
        }
        else
        {
            /* Running status: use last status */
            memset (txMidiBuffer + 2, 0, USBMIDI_LENGTH - 2);
            txMidiIndex = 0;
            txState = MIDITX_STATE_DATA;
        }
        retry = TRUE;
        break;

    case MIDITX_STATE_SYSTEM:
        if (data & MIDI_STATUS_BIT)
        {
            /* Expecting only End of Exclusive */
            if (data != MIDI_STATUS_END_SYSEX)
            {
                txState = MIDITX_STATE_ERROR;
                retry = TRUE;
            }
            else
            {
                /* Add buffer and return to idle */
                txMidiBuffer [USB_MIDI_DATA_OFFSET + txMidiIndex] = MIDI_STATUS_END_SYSEX;
                txMidiIndex = USBMIDI_CIN_SYS_END1 + txMidiIndex;

                /* Add appropriate CIN value (dependent on number of bytes being sent */
                txMidiBuffer [0] = (MIDI_CABLE << USBMIDI_CABLE_SHIFT) | (txMidiIndex << USBMIDI_CIN_SHIFT);
                AddMidiBuffer ();
                txState = MIDITX_STATE_IDLE;
            }
        }
        else
        {
            /* Normal data byte, add to buffer */
            txMidiBuffer [USB_MIDI_DATA_OFFSET + txMidiIndex] = data;
            txMidiIndex++;
            if (txMidiIndex >= (USBMIDI_LENGTH - USB_MIDI_DATA_OFFSET))
            {
                /* Filled buffer so complete */
                txMidiBuffer [0] = (MIDI_CABLE << USBMIDI_CABLE_SHIFT) | (USBMIDI_CIN_SYS_START << USBMIDI_CIN_SHIFT);
                AddMidiBuffer ();
                memset (txMidiBuffer, 0, USBMIDI_LENGTH);
                txMidiIndex = 0;
            }
        }
        break;

    case MIDITX_STATE_ERROR:
        txState = MIDITX_STATE_IDLE;
        txRunningStatus = 0;
        break;
    }

    return  retry;

}

/*
**  Add txMidiBuffer to (USB) output
*/
void AddMidiBuffer (void)
{
    if (txIndex < MIDI_IN_SIZE)
    {
        byte    i;

        for (i = 0; i < USBMIDI_LENGTH; ++i)
        {
            txMidiIn [txIndex + i] = txMidiBuffer [i];
        }
        txIndex += USBMIDI_LENGTH;
    }
    else
    {
        /* Buffer overrun: drop on floor */
        DEBUG_LED_ON;
    }
}

/**
**  Handle data received from USB for transmission to MIDI device
*/
void DepacketiseData (void)
{
    switch (rxState)
    {
    case MIDIRX_STATE_IDLE:
        if (!mMIDIUsartRxIsBusy())
        {
            /* Packet received: start processing it */
            rxState = MIDIRX_STATE_HEADER;
            rxIndex = 0;
        }
        break;

    case MIDIRX_STATE_HEADER:
        /* Any data remaining to be read? */
        if (rxIndex < MIDI_BULK_BD_OUT.Cnt)
        {
            /* Look for next chunk of data, skipping any weird lengths */
            if ((rxIndex + USBMIDI_LENGTH) <= MIDI_BULK_BD_OUT.Cnt)
            {
                byte    count;


                /* Find number of bytes in this USBMIDI packet */
                rxSubIndex = midi_data_rx [rxIndex] & USBMIDI_CIN_MASK;
                count = cinLength [rxSubIndex];
                if (0 == count)
                {
                    /* Zero means skip to end */
                    rxIndex = MIDI_BULK_BD_OUT.Cnt;
                }
                else
                {
                    /* Transfer data */
                    rxSubIndex = rxIndex + USB_MIDI_DATA_OFFSET;
                    rxLimit    = rxSubIndex + count;
                    rxState = MIDIRX_STATE_DATA;
                }
            }
            else
            {
                /* Abandon this transfer */
                rxIndex = MIDI_BULK_BD_OUT.Cnt;
            }
        }
        else
        {
            /*
            ** Completed: revert to idle and ownership reverts to USB
            */
            MIDI_BULK_BD_OUT.Cnt = sizeof(midi_data_rx);
            mUSBBufferReady(MIDI_BULK_BD_OUT);
            rxState = MIDIRX_STATE_IDLE;
        }
        break;

    case MIDIRX_STATE_DATA:
        /*
        ** Send data to MIDI device if UART is ready and DSP ready to receive
        */
        if (PIR1bits.TXIF)
        {
            if 	( LATBbits.LATB0 == 1) 	/* CTS from dsPIC   */
			{
				TXREG = midi_data_rx [rxSubIndex++];
            	if (rxSubIndex >= rxLimit)
            	{
                	rxIndex = rxLimit;
                	rxState = MIDIRX_STATE_HEADER;
            	}
			}
        }
        break;
    }
}



/**
**	InitialiseBootloader
**
**  Prepare system to run bootloader
**
*/

void InitialiseBootloader (void)
{
	/*	RTS to be set low to stop comms from dsPIC  */
	LATBbits.LATB1 = 0;

	/* reset USART  */
    TXSTA = 0x00;       // TX enable BRGH=1
    RCSTA = 0x00;       // continuous RX
    BAUDCON = 0x00;     // BRG16 = 1

	/* erase first part of program memory  */
	EraseBootByte();
	
	Reset();
}

/**
**	EraseBootByte
**
**  Erase the beginning of the program memory
**
*/

void EraseBootByte(void)
{

#define ProgramMemStart					0x001000 //Beginning of application program memory (not occupied by bootloader).  

	ClrWdt();
	TBLPTR = ((unsigned short long)ProgramMemStart );
	EECON1 = 0b10010100;	//Prepare for erasing flash memory
	UnlockAndActivate();
}

/**
**	EraseBootByte
**
**  Erase the beginning of the program memory
**
*/

void UnlockAndActivate(void)
{
	INTCONbits.GIE = 0;		//Make certain interrupts disabled for unlock process.
	_asm
	//Now unlock sequence to set WR (make sure interrupts are disabled before executing this)
	MOVLW 0x55
	MOVWF EECON2, 0
	MOVLW 0xAA
	MOVWF EECON2, 0
	BSF EECON1, 1, 0		//Performs write
	_endasm	
	while(EECON1bits.WR);	//Wait until complete (relevant when programming EEPROM, not important when programming flash since processor stalls during flash program)	
	EECON1bits.WREN = 0;  	//Good practice now to clear the WREN bit, as further protection against any accidental activation of self write/erase operations.
}


/** EOF user.c ***************************************************************/
