/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        usbdsc.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ********************************************************************/

/*********************************************************************
 * -usbdsc.c-
 * This file contains the USB descriptor information. It is used
 * in conjunction with the usbdsc.h file. When a descriptor is added
 * or removed from the main configuration descriptor, i.e. CFG01,
 * the user must also change the descriptor structure defined in
 * the usbdsc.h file. The structure is used to calculate the
 * descriptor size, i.e. sizeof(CFG01).
 *
 * A typical configuration descriptor consists of:
 * At least one configuration descriptor (USB_CFG_DSC)
 * One or more interface descriptors (USB_INTF_DSC)
 * One or more endpoint descriptors (USB_EP_DSC)
 *
 * Naming Convention:
 * To resolve ambiguity, the naming convention are as followed:
 * - USB_CFG_DSC type should be named cdxx, where xx is the
 *   configuration number. This number should match the actual
 *   index value of this configuration.
 * - USB_INTF_DSC type should be named i<yy>a<zz>, where yy is the
 *   interface number and zz is the alternate interface number.
 * - USB_EP_DSC type should be named ep<##><d>_i<yy>a<zz>, where
 *   ## is the endpoint number and d is the direction of transfer.
 *   The interface name should also be listed as a suffix to identify
 *   which interface does the endpoint belong to.
 *
 * Example:
 * If a device has one configuration, two interfaces; interface 0
 * has two endpoints (in and out), and interface 1 has one endpoint(in).
 * Then the CFG01 structure in the usbdsc.h should be:
 *
 * #define CFG01 rom struct                            \
 * {   USB_CFG_DSC             cd01;                   \
 *     USB_INTF_DSC            i00a00;                 \
 *     USB_EP_DSC              ep01o_i00a00;           \
 *     USB_EP_DSC              ep01i_i00a00;           \
 *     USB_INTF_DSC            i01a00;                 \
 *     USB_EP_DSC              ep02i_i01a00;           \
 * } cfg01
 *
 * Note the hierarchy of the descriptors above, it follows the USB
 * specification requirement. All endpoints belonging to an interface
 * should be listed immediately after that interface.
 *
 * -------------------------------------------------------------------
 * Filling in the descriptor values in the usbdsc.c file:
 * -------------------------------------------------------------------
 * Most items should be self-explanatory, however, a few will be
 * explained for clarification.
 *
 * [Configuration Descriptor(USB_CFG_DSC)]
 * The configuration attribute must always have the _DEFAULT
 * definition at the minimum. Additional options can be ORed
 * to the _DEFAULT attribute. Available options are _SELF and _RWU.
 * These definitions are defined in the usbdefs_std_dsc.h file. The
 * _SELF tells the USB host that this device is self-powered. The
 * _RWU tells the USB host that this device supports Remote Wakeup.
 *
 * [Endpoint Descriptor(USB_EP_DSC)]
 * Assume the following example:
 * sizeof(USB_EP_DSC),DSC_EP,_EP01_OUT,_BULK,64,0x00
 *
 * The first two parameters are self-explanatory. They specify the
 * length of this endpoint descriptor (7) and the descriptor type.
 * The next parameter identifies the endpoint, the definitions are
 * defined in usbdefs_std_dsc.h and has the following naming
 * convention:
 * _EP<##>_<dir>
 * where ## is the endpoint number and dir is the direction of
 * transfer. The dir has the value of either 'OUT' or 'IN'.
 * The next parameter identifies the type of the endpoint. Available
 * options are _BULK, _INT, _ISO, and _CTRL. The _CTRL is not
 * typically used because the default control transfer endpoint is
 * not defined in the USB descriptors. When _ISO option is used,
 * addition options can be ORed to _ISO. Example:
 * _ISO|_AD|_FE
 * This describes the endpoint as an isochronous pipe with adaptive
 * and feedback attributes. See usbdefs_std_dsc.h and the USB
 * specification for details. The next parameter defines the size of
 * the endpoint. The last parameter in the polling interval.
 *
 * -------------------------------------------------------------------
 * Adding a USB String
 * -------------------------------------------------------------------
 * A string descriptor array should have the following format:
 *
 * rom struct{byte bLength;byte bDscType;word string[size];}sdxxx={
 * sizeof(sdxxx),DSC_STR,<text>};
 *
 * The above structure provides a means for the C compiler to
 * calculate the length of string descriptor sdxxx, where xxx is the
 * index number. The first two bytes of the descriptor are descriptor
 * length and type. The rest <text> are string texts which must be
 * in the unicode format. The unicode format is achieved by declaring
 * each character as a word type. The whole text string is declared
 * as a word array with the number of characters equals to <size>.
 * <size> has to be manually counted and entered into the array
 * declaration. Let's study this through an example:
 * if the string is "USB" , then the string descriptor should be:
 * (Using index 02)
 * rom struct{byte bLength;byte bDscType;word string[3];}sd002={
 * sizeof(sd002),DSC_STR,'U','S','B'};
 *
 * A USB project may have multiple strings and the firmware supports
 * the management of multiple strings through a look-up table.
 * The look-up table is defined as:
 * rom const unsigned char *rom USB_SD_Ptr[]={&sd000,&sd001,&sd002};
 *
 * The above declaration has 3 strings, sd000, sd001, and sd002.
 * Strings can be removed or added. sd000 is a specialized string
 * descriptor. It defines the language code, usually this is
 * US English (0x0409). The index of the string must match the index
 * position of the USB_SD_Ptr array, &sd000 must be in position
 * USB_SD_Ptr[0], &sd001 must be in position USB_SD_Ptr[1] and so on.
 * The look-up table USB_SD_Ptr is used by the get string handler
 * function in usb9.c.
 *
 * -------------------------------------------------------------------
 *
 * The look-up table scheme also applies to the configuration
 * descriptor. A USB device may have multiple configuration
 * descriptors, i.e. CFG01, CFG02, etc. To add a configuration
 * descriptor, user must implement a structure similar to CFG01.
 * The next step is to add the configuration descriptor name, i.e.
 * cfg01, cfg02,.., to the look-up table USB_CD_Ptr. USB_CD_Ptr[0]
 * is a dummy place holder since configuration 0 is the un-configured
 * state according to the definition in the USB specification.
 *
 ********************************************************************/

/*********************************************************************
 * Descriptor specific type definitions are defined in:
 * system\usb\usbdefs\usbdefs_std_dsc.h
 *
 * Configuration information is defined in:
 * autofiles\usbcfg.h
 ********************************************************************/

/** I N C L U D E S *************************************************/
#include "system\typedefs.h"

#include "system\usb\class\MidiStream.h"

#include "system\usb\usb.h"


/** C O N S T A N T S ************************************************/
#ifndef HI_TECH_C
#pragma romdata
#endif

#define VENDOR_ID           (0x1D17)    /* C-Thru */
#define PRODUCT_ID_HTMINI   (0x0001)    /* HT Mini */
#define PRODUCT_ID_AXIS64   (0x0002)    /* AXIS-64 */

/* Device Descriptor */
rom USB_DEV_DSC device_dsc=
{
    sizeof(USB_DEV_DSC),    // Size of this descriptor in bytes
    DSC_DEV,                // DEVICE descriptor type
    0x0200,                 // USB Spec Release Number in BCD format
    0x00,                   // Class Code
    0x00,                   // Subclass code
    0x00,                   // Protocol code
    EP0_BUFF_SIZE,          // Max packet size for EP0, see usbcfg.h
    VENDOR_ID,              // Vendor ID
    PRODUCT_ID_HTMINI,      // Product ID
    0x0001,                 // Device release number in BCD format
    0x01,                   // Manufacturer string index
    0x02,                   // Product string index
    0x03,                   // Device serial number string index
    0x01                    // Number of possible configurations
};

/* Device Descriptor */
rom USB_DEV_DSC device_dsc_alt=
{
    sizeof(USB_DEV_DSC),    // Size of this descriptor in bytes
    DSC_DEV,                // DEVICE descriptor type
    0x0200,                 // USB Spec Release Number in BCD format
    0x00,                   // Class Code
    0x00,                   // Subclass code
    0x00,                   // Protocol code
    EP0_BUFF_SIZE,          // Max packet size for EP0, see usbcfg.h
    VENDOR_ID,              // Vendor ID
    PRODUCT_ID_AXIS64,      // Product ID
    0x0001,                 // Device release number in BCD format
    0x01,                   // Manufacturer string index
    0x02,                   // Product string index
    0x03,                   // Device serial number string index
    0x01                    // Number of possible configurations
};


CFG01=
{
    /* Configuration Descriptor */
    sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
    DSC_CFG,                // CONFIGURATION descriptor type
    sizeof(cfg01),          // Total length of data for this cfg
    1,                      // Number of interfaces in this cfg //    // \todo add programming interface
    1,                      // Index value of this configuration
    0,                      // Configuration string index
    _DEFAULT,               // Attributes, see usbdefs_std_dsc.h
    50,                     // Max power consumption (2X mA)

    // STANDARD MIDI STREAMING (MS) INTERFACE DESCRIPTOR
    sizeof(STANDARD_MS_INTERFACE_DESCRIPTOR), // Size of this descriptor in bytes
    DSC_INTF,               // INTERFACE descriptor type
    0,                      // Index of this interface
    0,                      // Index of Alternate Setting
    2,                      // Number of end points
    _AUDIO,                 // Interface Class
    _MIDISTREAMING,         // Interface Sub Class
    0,                      // Interface Protocol (Unused)
    0,                      // Inteface (Unused)

    // CLASS SPECIFIC MIDI STREAMING (MS) INTERFACE DESCRIPTOR
    sizeof(CS_MS_INTERFACE_DESCRIPTOR), // Size of this descriptor in bytes
    _CS_INTERFACE,          // INTERFACE descriptor type
    _MS_HEADER,             // Descriptor Sub-Type
    0x0100,                 // Revision of class specification
    sizeof(CS_MS_INTERFACE_DESCRIPTOR)+  //Total size of class specific descritors
    (2 * sizeof(MIDI_ADAPTER_MIDI_IN_JACK_DESCRIPTOR)) + // Size of MIDI IN  JACK descriptor in bytes
    (2 * sizeof(MIDI_ADAPTER_MIDI_OUT_JACK_DESCRIPTOR)) + // Size of MIDI OUT JACK this descriptor in bytes
    (2 * sizeof(USB_EP_DSC)) +
    (2 * sizeof(USB_CS_MS_EP_DSC)),

    // MIDI Adapter MIDI IN Jack Descriptor (Embedded)
    sizeof(MIDI_ADAPTER_MIDI_IN_JACK_DESCRIPTOR), // Size of this descriptor in bytes
    _CS_INTERFACE,          // INTERFACE descriptor type
    _MIDI_IN_JACK,          // Descriptor Sub-Type
    _EMBEDDED,              // Jack Type
    0x01,                    // ID of this Jack
    0,                      // Unused \todo Add Index to string descriptor here

    // MIDI Adapter MIDI IN Jack Descriptor (External)
    sizeof(MIDI_ADAPTER_MIDI_IN_JACK_DESCRIPTOR), // Size of this descriptor in bytes
    _CS_INTERFACE,          // INTERFACE descriptor type
    _MIDI_IN_JACK,          // Descriptor Sub-Type
    _EXTERNAL,              // Jack Type
    0x02,                    // ID of this Jack
    0,                      // Unused \todo Add Index to string descriptor here


    // MIDI Adapter MIDI OUT Jack Descriptor (Embedded)
    sizeof(MIDI_ADAPTER_MIDI_OUT_JACK_DESCRIPTOR), // Size of this descriptor in bytes
    _CS_INTERFACE,          // INTERFACE descriptor type
    _MIDI_OUT_JACK,         // Descriptor Sub-Type
    _EMBEDDED,              // Jack Type
    0x03,                    // ID of this Jack
    0x01,                   // Number of input pins on this Jack
    0x02,                   // ID of the entity to which this pin is connected
    0x01,                   // Output Pin Number of the entity to which this pin is connected
    0,                       // Unused \todo Add Index to string descriptor here

    // MIDI Adapter MIDI OUT Jack Descriptor (external)
    sizeof(MIDI_ADAPTER_MIDI_OUT_JACK_DESCRIPTOR), // Size of this descriptor in bytes
    _CS_INTERFACE,          // INTERFACE descriptor type
    _MIDI_OUT_JACK,         // Descriptor Sub-Type
    _EXTERNAL,              // Jack Type
    0x04,                   // ID of this Jack
    0x01,                   // Number of input pins on this Jack
    0x01,                   // ID of the entity to which this pin is connected
    0x01,                   // Output Pin Number of the entity to which this pin is connected
    0,                      // Unused \todo Add Index to string descriptor here

    /* Endpoint Descriptors */
    // Standard Bulk OUT Endpoint Descriptor
    sizeof(USB_EP_DSC),     // Size of this descriptor in bytes
    DSC_EP,                 // Descriptor Sub-Type
    _EP01_OUT,              // End point address
    _BULK,                  // bulk attribute
    64,                     // 64 bytes per package
    0,                      // interval - ignored for bulk
    0,                      // refresh - unused
    0,                      // synch address - unused

    // Class Specific MS Bulk OUT Endpoint Descriptor
    sizeof(USB_CS_MS_EP_DSC),   // Size of this descriptor in bytes
    _CS_ENDPOINT,               // Descriptor Type
    _MS_GENERAL,                // Descriptor Sub-Type
    1,                          // number of embedded MIDI IN jacks
    0x01,                       // ID of Embedded MIDI IN jack

    // Standard Bulk IN Endpoint Descriptor
    sizeof(USB_EP_DSC),     // Size of this descriptor in bytes
    DSC_EP,                 // Descriptor Sub-Type
    _EP01_IN,               // End point address
    _BULK,                  // bulk attribute
    64,                     // 64 bytes per package
    0,                      // interval - ignored for bulk
    0,                      // refresh - unused
    0,                      // synch address - unused

    // Class Specific MS Bulk IN Endpoint Descriptor
    sizeof(USB_CS_MS_EP_DSC),   // Size of this descriptor in bytes
    _CS_ENDPOINT,               // Descriptor Type
    _MS_GENERAL,                // Descriptor Sub-Type
    1,                          // number of embedded MIDI OUT jacks
    0x03                        // ID of Embedded MIDI OUT jack
};



rom struct{byte bLength;byte bDscType;word string[1];}sd000={
sizeof(sd000),DSC_STR,0x0409};

rom struct{byte bLength;byte bDscType;word string[16];}sd001={
sizeof(sd001),DSC_STR,
'C','-','T','h','r','u',' ','M','u','s','i','c',' ','L','t','d'};

rom struct{byte bLength;byte bDscType;word string[10];}sd002={  // CHANGE from [20] Version 0.12
sizeof(sd002),DSC_STR,
//'A','X','I','S','-','4','9',' ','U','S','B',' ','K','e','y','b','o','a','r','d'};  //DEL Version 0.12
'A','X','I','S','-','4','9',' ','2','A'};        //ADD Version 0.12

rom struct{byte bLength;byte bDscType;word string[10];}sd003={  // CHANGE from [20] Version 0.12
sizeof(sd003),DSC_STR,
//'A','X','I','S','-','6','4',' ','U','S','B',' ','K','e','y','b','o','a','r','d'};  //DEL Version 0.12
'A','X','I','S','-','6','4',' ','X','X'};   //ADD Version 0.12

rom struct{byte bLength;byte bDscType;word string[20];}sd004={
sizeof(sd004),DSC_STR,
'C','T','M','A','X','4','9',' ',' ',' ',' ',' ','0','0','.','0','0','.','1','3'};   //CHA version number from 12 Version 0.13

rom struct{byte bLength;byte bDscType;word string[20];}sd005={
sizeof(sd005),DSC_STR,
'C','T','M','A','X','6','4',' ',' ',' ',' ',' ','0','0','.','0','0','.','0','0'};

const_rom unsigned char *rom USB_CD_Ptr[]=
{
    (const_rom unsigned char *) &cfg01
};

const_rom unsigned char *rom USB_SD_Ptr[]=
{
    (const_rom unsigned char *) &sd000,
    (const_rom unsigned char *) &sd001,
    (const_rom unsigned char *) &sd002,
    (const_rom unsigned char *) &sd004
};

const_rom unsigned char *rom USB_SD_alt_Ptr[]=
{
    (const_rom unsigned char *) &sd000,
    (const_rom unsigned char *) &sd001,
    (const_rom unsigned char *) &sd003,
    (const_rom unsigned char *) &sd005
};

rom pFunc ClassReqHandler[1]=
{
    USBCheckMIDIRequest
};

/** EOF usbdsc.c ****************************************************/
