/**
 *
 * @file MidiStream.c
 *
 * Filename          : MidiStream.c
 *
 * Module            : MidiStream.c
 *
 * Author            : Val Tarrant
 *
 * Description       : file with specific USB declarations for midi stream
 *
 *
 * Change history    :
 *
 *   Issue   Date        Changed by      Reason for change
 *
 *   0.01    31Oct2007   V A Tarrant     First Issue
 *   0.02    20Nov2007   J G Thorne      Add USB-MIDI endpoint initialisation
 *
 *
 *
 * Copyright         : Cambridge Micro Engineering, 2007
 */

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"

#include "MIDIStream.h"

/** V A R I A B L E S ********************************************************/

#pragma udata
byte midi_rx_len;            // total rx length

byte midi_trf_state;         // States are defined midistream.h
POINTER pMIDISrc;            // Dedicated source pointer
POINTER pMIDIDst;            // Dedicated destination pointer
byte midi_tx_len;            // total tx length
byte midi_mem_type;          // _ROM, _RAM

/** D E C L A R A T I O N S **************************************************/
#pragma code


/** C L A S S  S P E C I F I C  R E Q ****************************************/
/******************************************************************************
 * Function:        void USBCheckMIDIRequest(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine checks the setup data packet to see if it
 *                  knows how to handle it
 *
 * Note:            None
 *****************************************************************************/
void USBCheckMIDIRequest(void)
{
   /*
     * If request recipient is not an interface then return
     */
    if(SetupPkt.Recipient != RCPT_INTF) return;

    /*
     * If request type is not class-specific then return
     */
    if(SetupPkt.RequestType != CLASS) return;

    /*
     * Interface ID must match interface numbers associated with
     * MIDI class, else return
     */
    if(SetupPkt.bIntfID != MIDI_DATA_INTF_ID) return;

    switch(SetupPkt.bRequest)
    {
        /* No MIDI commands to handle at present */
    default:
        break;
    }//end switch(SetupPkt.bRequest)


}//end USBCheckMIDIRequest

/** U S E R  A P I ***********************************************************/

/******************************************************************************
 * Function:        void MIDIInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        MIDIInitEP initializes MIDI endpoints, buffer descriptors,
 *                  internal state-machine, and variables.
 *                  It should be called after the USB host has sent out a
 *                  SET_CONFIGURATION request.
 *                  See USBStdSetCfgHandler() in usb9.c for examples.
 *
 * Note:            None
 *****************************************************************************/
void MIDIInitEP(void)
{
    midi_trf_state = MIDI_TX_READY;
    midi_rx_len = 0;

    MIDI_DATA_UEP = EP_OUT_IN|HSHK_EN;           // Enable 2 data pipes

    /*
     * Do not have to init Cnt of IN pipes here.
     * Reason:  Number of bytes to send to the host
     *          varies from one transaction to
     *          another. Cnt should equal the exact
     *          number of bytes to transmit for
     *          a given IN transaction.
     *          This number of bytes will only
     *          be known right before the data is
     *          sent.
     */

    MIDI_BULK_BD_OUT.Cnt = sizeof(midi_data_rx);  // Set buffer size
    MIDI_BULK_BD_OUT.ADR = (byte*)midi_data_rx;  // Set buffer address
    MIDI_BULK_BD_OUT.Stat._byte = _USIE|_DAT0|_DTSEN;// Set status

    MIDI_BULK_BD_IN.ADR = (byte*)midi_data_tx;   // Set buffer size
    MIDI_BULK_BD_IN.Stat._byte = _UCPU|_DAT1;    // Set buffer address

}//end MIDIInitEP

/******************************************************************************
 * Function:        void MIDITxService(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        MIDITxService handles device-to-host transaction(s).
 *                  This function should be called once per Main Program loop.
 *
 * Note:            None
 *****************************************************************************/
void MIDITxService(void)
{

    byte byte_to_send;

    /*
     * No point in processing if Tx endpoint not ready
     */
    if(mMIDIUsartTxIsBusy()) return;

    /*
     * Completing stage is necessary while [ mMIDIUsartTxIsBusy()==1 ].
     * By having this stage, user can always check midi_trf_state,
     * and not having to call mMIDIUsartTxIsBusy() directly.
     */
    if(midi_trf_state == MIDI_TX_COMPLETING)
        midi_trf_state = MIDI_TX_READY;

    /*
     * If MIDI_TX_READY state, nothing to do, just return.
     */
    if(midi_trf_state == MIDI_TX_READY) return;

    /*
     * If MIDI_TX_BUSY_ZLP state, send zero length packet
     */
    if(midi_trf_state == MIDI_TX_BUSY_ZLP)
    {
        MIDI_BULK_BD_IN.Cnt = 0;
        midi_trf_state = MIDI_TX_COMPLETING;
    }
    else if(midi_trf_state == MIDI_TX_BUSY)
    {
        /*
         * First, have to figure out how many byte of data to send.
         */
        if(midi_tx_len > sizeof(midi_data_tx))
            byte_to_send = sizeof(midi_data_tx);
        else
            byte_to_send = midi_tx_len;

        /*
         * Next, load the number of bytes to send to Cnt in buffer descriptor
         */
        MIDI_BULK_BD_IN.Cnt = byte_to_send;

        /*
         * Subtract the number of bytes just about to be sent from the total.
         */
        midi_tx_len = midi_tx_len - byte_to_send;

        pMIDIDst.bRam = (byte*)&midi_data_tx; // Set destination pointer

        if(midi_mem_type == _ROM)            // Determine type of memory source
        {
            while(byte_to_send)
            {
                *pMIDIDst.bRam = *pMIDISrc.bRom;
                pMIDIDst.bRam++;
                pMIDISrc.bRom++;
                byte_to_send--;
            }//end while(byte_to_send)
        }
        else // _RAM
        {
            while(byte_to_send)
            {
                *pMIDIDst.bRam = *pMIDISrc.bRam;
                pMIDIDst.bRam++;
                pMIDISrc.bRam++;
                byte_to_send--;
            }//end while(byte_to_send._word)
        }//end if(midi_mem_type...)

        /*
         * Lastly, determine if a zero length packet state is necessary.
         * See explanation in USB Specification 2.0: Section 5.8.3
         */
        if(midi_tx_len == 0)
        {
            if(MIDI_BULK_BD_IN.Cnt == sizeof(midi_data_tx))
                midi_trf_state = MIDI_TX_BUSY_ZLP;
            else
                midi_trf_state = MIDI_TX_COMPLETING;
        }//end if(midi_tx_len...)

    }//end if(midi_tx_sate == MIDI_TX_BUSY)

    /*
     * Both MIDI_TX_BUSY and MIDI_TX_BUSY_ZLP states use the following macro
     */
    mUSBBufferReady(MIDI_BULK_BD_IN);

}//end MIDITxService

/**
**  Transmit MIDI data to USB IN pipe.  The data to be
**  transferred must be in USB-MIDI format.
**
**  @param  data    Pointer to data (RAM)
**  @param  length  Length of data
**
**  @retval bool
**              -   TRUE    Data taken
**              -   FALSE   Data not taken (please call again)
*/
BOOL MidiTxData(byte *data, byte length)
{
    BOOL    rc = FALSE;

    if (mUSBUSARTIsTxTrfReady())
    {
        mUSBUSARTTxRam ((byte *) data, length);
        rc = TRUE;
    }

    return  rc;
}
