/**
 *
 * @file MidiStream.h
 *
 * Filename          : MidiStream.h
 *
 * Module            :
 *
 * Author            : Val Tarrant
 *
 * Description       : Header file with specific USB declarations for midi stream
 *
 *
 * Change history    :
 *
 *   Issue   Date        Changed by      Reason for change
 *
 *   0.01    31Oct2007   V A Tarrant     First Issue
 *
 *
 *
 * Copyright         : Cambridge Micro Engineering, October 2007
 */

#ifndef __MIDISTREAM_H__
    #define __MIDISTREAM_H__

#include "system\typedefs.h"

/* Audio interface class code */
#define _AUDIO         0x01

/* Audion interface subclass codes */
#define _SUBCLASS_UNDEFINED 0x00
#define _AUDIOCONTROL       0x01
#define _AUDIOSTREAMING     0x02
#define _MIDISTREAMING      0x03

/* audio class specific descriptor types */
#define _CS_UNDEFINED       0x20
#define _CS_DEVICE          0x21
#define _CS_CONFIGURATION   0x22
#define _CS_STRING          0x23
#define _CS_INTERFACE       0x24
#define _CS_ENDPOINT        0x25

/* Audio Class-Specific AC Interface Descriptor Subtypes*/
#define _HEADER         0x01

/* MS Class specific Interface descriptor types */
#define _MS_DESCRIPTOR_UNDEFINED 0x00
#define _MS_HEADER               0x01
#define _MIDI_IN_JACK            0x02
#define _MIDI_OUT_JACK           0x03
#define _ELEMENT                 0x04

/* MS class specific endpoint descriptor subtypes */
#define _DESCRIPTOR_UNDEFINED    0x00
#define _MS_GENERAL              0x01

/* MS Midi in and out jack types */
#define _JACK_TYPE_UNDEFINED     0x00
#define _EMBEDDED                0x01
#define _EXTERNAL                0x02

/* Midi Bulk IN transfer states */
#define MIDI_TX_READY                0
#define MIDI_TX_BUSY                 1
#define MIDI_TX_BUSY_ZLP             2       // ZLP: Zero Length Packet
#define MIDI_TX_COMPLETING           3

/** S T R U C T U R E S ******************************************************/
typedef struct _CS_AC_INTERFACE_HEADER_DESCRIPTOR
{
    byte bLength;   byte bDescriptorType;   byte bDescriptorSubtype;
    word bcdADC;    word wTotalLength;      byte bInCollection;
    byte baInterfaceNr;
} CS_AC_INTERFACE_HEADER_DESCRIPTOR;

typedef struct _STANDARD_MS_INTERFACE_DESCRIPTOR
{
    byte bLength;            byte bDescriptorType;  byte bInterfaceNumber;
    byte bAlternateSetting;  byte bNumEndpoints;    byte bInterfaceClass;
    byte bInterfaceSubClass; byte bInterfaceProt;   byte bInterface;
} STANDARD_MS_INTERFACE_DESCRIPTOR;

typedef struct _CS_MS_INTERFACE_DESCRIPTOR
{
    byte bLength;   byte bDescriptorType;   byte bDescriptorSubtype;
    word BcdADC;    word wTotalLength;
} CS_MS_INTERFACE_DESCRIPTOR;

typedef struct _MIDI_ADAPTER_MIDI_IN_JACK_DESCRIPTOR
{
    byte bLength;       byte bDescriptorType;   byte bDescriptorSubtype;
    byte bJacktype;     byte bJackID;           byte iJack;
} MIDI_ADAPTER_MIDI_IN_JACK_DESCRIPTOR;

typedef struct _MIDI_ADAPTER_MIDI_OUT_JACK_DESCRIPTOR
{
    byte bLength;       byte bDescriptorType;   byte bDescriptorSubtype;
    byte bJacktype;     byte bJackID;           byte bNrInputPins;
    byte BaSourceID;    byte BaSourcePin;       byte iJack;
} MIDI_ADAPTER_MIDI_OUT_JACK_DESCRIPTOR;

typedef struct _USB_CS_MS_EP_DSC
{
    byte bLength;           byte bDescriptorType;   byte bDescriptorSubtype;
    byte bNumEmbMIDIJack;   byte BaAssocJackID;
} USB_CS_MS_EP_DSC;


/* Line Coding Structure */
#define LINE_CODING_LENGTH          0x07

typedef struct _LINE_CODING
{
    byte _byte[LINE_CODING_LENGTH];
} LINE_CODING;

typedef union _CONTROL_SIGNAL_BITMAP
{
    byte _byte;
    struct
    {
        unsigned DTE_PRESENT;       // [0] Not Present  [1] Present
        unsigned CARRIER_CONTROL;   // [0] Deactivate   [1] Activate
    } a;
} CONTROL_SIGNAL_BITMAP;

/******************************************************************************
 * Macro:           BOOL mUSBUSARTIsTxTrfReady(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if the MIDI class is ready
 *                  to send more data.
 *                  Typical Usage: if(mUSBUSARTIsTxTrfReady())
 *
 * Note:            None
 *****************************************************************************/
#define mUSBUSARTIsTxTrfReady()     (midi_trf_state == MIDI_TX_READY)

/******************************************************************************
 * Macro:           (bit) mMIDIUsartRxIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if MIDI bulk OUT endpoint is
 *                  busy (owned by SIE) or not.
 *                  Typical Usage: if(mMIDIUsartRxIsBusy())
 *
 * Note:            None
 *****************************************************************************/
#define mMIDIUsartRxIsBusy()         MIDI_BULK_BD_OUT.Stat.UOWN

/******************************************************************************
 * Macro:           (bit) mMIDIUsartTxIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if MIDI bulk IN endpoint is
 *                  busy (owned by SIE) or not.
 *                  Typical Usage: if(mMIDIUsartTxIsBusy())
 *
 * Note:            None
 *****************************************************************************/
#define mMIDIUsartTxIsBusy()         MIDI_BULK_BD_IN.Stat.UOWN

/******************************************************************************
 * Macro:           byte mMIDIGetRxLength(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          mMIDIGetRxLength returns midi_rx_len
 *
 * Side Effects:    None
 *
 * Overview:        mMIDIGetRxLength is used to retrieve the number of bytes
 *                  copied to user's buffer by the most recent call to
 *                  getsUSBUSART function.
 *
 * Note:            None
 *****************************************************************************/
#define mMIDIGetRxLength()           midi_rx_len

/******************************************************************************
 * Macro:           void mUSBUSARTTxRam(byte *pData, byte len)
 *
 * PreCondition:    mdid_trf_state must be in the MIDI_TX_READY state.
 *
 *                  Value of 'len' must be equal to or smaller than 255 bytes.
 *
 * Input:           pDdata  : Pointer to the starting location of data bytes
 *                  len     : Number of bytes to be transferred
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Use this macro to transfer data located in data memory.
 *                  Use this macro when:
 *                  1. Data stream is not null-terminated
 *                  2. Transfer length is known
 *
 *                  Remember: midi_trf_state must == MIDI_TX_READY
 *                  Unlike putsUSBUSART, there is not code double checking
 *                  the transfer state. Unexpected behavior will occur if
 *                  this function is called when midi_trf_state != MIDI_TX_READY
 *
 * Note:            This macro only handles the setup of the transfer. The
 *                  actual transfer is handled by MIDITxService().
 *****************************************************************************/
#define mUSBUSARTTxRam(pData,len)   \
{                                   \
    pMIDISrc.bRam = pData;           \
    midi_tx_len = len;               \
    midi_mem_type = _RAM;            \
    midi_trf_state = MIDI_TX_BUSY;    \
}

/******************************************************************************
 * Macro:           void mUSBUSARTTxRom(rom byte *pData, byte len)
 *
 * PreCondition:    midi_trf_state must be in the MIDI_TX_READY state.
 *
 *                  Value of 'len' must be equal to or smaller than 255 bytes.
 *
 * Input:           pDdata  : Pointer to the starting location of data bytes
 *                  len     : Number of bytes to be transferred
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Use this macro to transfer data located in program memory.
 *                  Use this macro when:
 *                  1. Data stream is not null-terminated
 *                  2. Transfer length is known
 *
 *                  Remember: midi_trf_state must == MIDI_TX_READY
 *                  Unlike putrsUSBUSART, there is not code double checking
 *                  the transfer state. Unexpected behavior will occur if
 *                  this function is called when midi_trf_state != MIDI_TX_READY
 *
 * Note:            This macro only handles the setup of the transfer. The
 *                  actual transfer is handled by MIDITxService().
 *****************************************************************************/
#define mUSBUSARTTxRom(pData,len)   \
{                                   \
    pMIDISrc.bRom = pData;           \
    midi_tx_len = len;               \
    midi_mem_type = _ROM;            \
    midi_trf_state = MIDI_TX_BUSY;    \
}


/** P U B L I C  P R O T O T Y P E S *****************************************/
void USBCheckMIDIRequest(void);
void MIDIInitEP(void);
void MIDITxService(void);
void MIDIRxService(void);
BOOL MidiTxData(byte *data, byte length);

#endif  /*  #ifndef __MIDISTREAM_H__  */